package com.abi.platform.api;


import com.abi.platform.Base.BaseResponse;
import com.abi.platform.dto.req.BatchImportBrandReq;
import com.abi.platform.dto.req.BatchImportCoverFactoryReq;
import com.abi.platform.dto.req.BatchImportMaterialReq;
import com.abi.platform.dto.req.BatchImportOrganizationReq;
import com.abi.platform.dto.req.BatchImportPackageSapOrderItemReq;
import com.abi.platform.dto.req.BatchImportPackageSapOrderReq;
import com.abi.platform.dto.req.BatchImportSapBeerFactoryReq;
import com.abi.platform.dto.req.BatchImportSkuReq;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

/**
 * @author: fangxinjian
 * @date: 2021/06/02 11:26
 * @description: sap导入数据
 */
@RequestMapping("/sap")
public interface BaseSapImportApi {

    /**
     * 批量导入组织架构数据
     *
     * @param batchImportOrganizationReq 批量导入组织架构数据入参
     * @return
     */
    @PostMapping("batchImportOrganization")
    BaseResponse batchImportOrganization(@RequestBody BatchImportOrganizationReq batchImportOrganizationReq);

    /**
     * 批量修改组织架构数据
     *
     * @param batchImportOrganizationReq 批量修改组织架构数据入参
     * @return
     */
    @PostMapping("batchUpdateOrganization")
    BaseResponse batchUpdateOrganization(@RequestBody BatchImportOrganizationReq batchImportOrganizationReq);

    /**
     * 批量导入SKU数据
     *
     * @param batchImportSkuReq 批量导入SKU数据入参
     * @return
     */
    @PostMapping("batchImportSku")
    BaseResponse batchImportSku(@RequestBody BatchImportSkuReq batchImportSkuReq);

    /**
     * 批量修改SKU数据
     *
     * @param batchImportSkuReq 批量修改SKU数据入参
     * @return
     */
    @PostMapping("batchUpdateSku")
    BaseResponse batchUpdateSku(@RequestBody BatchImportSkuReq batchImportSkuReq);

    /**
     * 批量导入品牌数据
     *
     * @param batchImportBrandReq 批量导入品牌数据入参
     * @return
     */
    @PostMapping("batchImportBrand")
    BaseResponse batchImportBrand(@RequestBody BatchImportBrandReq batchImportBrandReq);

    /**
     * 批量修改品牌数据
     *
     * @param batchImportBrandReq 批量修改品牌数据入参
     * @return
     */
    @PostMapping("batchUpdateBrand")
    BaseResponse batchUpdateBrand(@RequestBody BatchImportBrandReq batchImportBrandReq);

    /**
     * 批量导入Sap物料数据
     *
     * @param batchImportMaterialReq 批量导入Sap物料数据入参
     * @return
     */
    @PostMapping("batchImportMaterial")
    BaseResponse batchImportMaterial(@RequestBody BatchImportMaterialReq batchImportMaterialReq);

    /**
     * 批量修改Sap物料数据
     *
     * @param batchImportMaterialReq 批量修改Sap物料数据入参
     * @return
     */
    @PostMapping("batchUpdateMaterial")
    BaseResponse batchUpdateMaterial(@RequestBody BatchImportMaterialReq batchImportMaterialReq);

    /**
     * 批量导入Sap供应商数据
     *
     * @param batchImportCoverFactoryReq 批量导入Sap供应商数据入参
     * @return
     */
    @PostMapping("batchImportCoverFactory")
    BaseResponse batchImportCoverFactory(@RequestBody BatchImportCoverFactoryReq batchImportCoverFactoryReq);

    /**
     * 批量修改Sap供应商数据
     *
     * @param batchImportCoverFactoryReq 批量修改Sap供应商数据入参
     * @return
     */
    @PostMapping("batchUpdateCoverFactory")
    BaseResponse batchUpdateCoverFactory(@RequestBody BatchImportCoverFactoryReq batchImportCoverFactoryReq);

    /**
     * 批量导入Sap啤酒厂数据
     *
     * @param batchImportSapBeerFactoryReq 批量导入Sap啤酒厂数据入参
     * @return
     */
    @PostMapping("batchImportSapBeerFactory")
    BaseResponse batchImportSapBeerFactory(@RequestBody BatchImportSapBeerFactoryReq batchImportSapBeerFactoryReq);

    /**
     * 批量修改Sap啤酒厂数据
     *
     * @param batchImportSapBeerFactoryReq 批量修改Sap啤酒厂数据入参
     * @return
     */
    @PostMapping("batchUpdateSapBeerFactory")
    BaseResponse batchUpdateSapBeerFactory(@RequestBody BatchImportSapBeerFactoryReq batchImportSapBeerFactoryReq);

    /**
     * 批量导入Sap码包订单数据
     *
     * @param batchImportPackageSapOrderReq 批量导入Sap码包订单数据入参
     * @return
     */
    @PostMapping("batchImportPackageSapOrder")
    BaseResponse batchImportPackageSapOrder(@RequestBody BatchImportPackageSapOrderReq batchImportPackageSapOrderReq);

    /**
     * 批量修改Sap码包订单数据
     *
     * @param batchImportPackageSapOrderReq 批量修改Sap码包订单数据入参
     * @return
     */
    @PostMapping("batchUpdatePackageSapOrder")
    BaseResponse batchUpdatePackageSapOrder(@RequestBody BatchImportPackageSapOrderReq batchImportPackageSapOrderReq);

    /**
     * 批量导入Sap物料订单明细数据
     *
     * @param batchImportPackageSapOrderItemReq 批量导入Sap物料订单明细数据入参
     * @return
     */
    @PostMapping("batchImportPackageSapOrderItem")
    BaseResponse batchImportPackageSapOrderItem(@RequestBody BatchImportPackageSapOrderItemReq batchImportPackageSapOrderItemReq);

    /**
     * 批量修改SAP物料订单明细数据
     *
     * @param batchImportPackageSapOrderItemReq 批量修改SAP物料订单明细数据入参
     * @return
     */
    @PostMapping("batchUpdatePackageSapOrderItem")
    BaseResponse batchUpdatePackageSapOrderItem(@RequestBody BatchImportPackageSapOrderItemReq batchImportPackageSapOrderItemReq);
}