package com.abi.task.common.api.exception;


import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.EnumSet;
import java.util.Set;

/**
 * @author:Andy.Tan
 * @Description: 错误返回枚举
 */
@Getter
@AllArgsConstructor
public enum ErrorCodeEnum {
	/**
	 *	执行成功
	 */
	SUCCESS(200, "执行成功"),
	/**
	 *	服务异常
	 */
	FAIL(999, "服务异常"),
	/**
	 *	参数错误
	 */
	ERROR_PARAM(771, "参数错误"),
	/**
	 *	请求失效
	 */
	REQ_EXPIRE(772, "请求失效"),
	/**
	 *	未知异常
	 */
	EXCEPT_UN_KNOW(773, "未知异常"),
	/**
	 *	数据库操作异常
	 */
	EXCEPT_DB_OPERATION(774, "数据库操作异常"),
	/**
	 *	服务异常
	 */
	SERVICE_EXCEPTION(775, "服务异常"),
	/**
	 *	用户不存在
	 */
	NO_EXISTS_USER(776, "用户不存在"),
	/**
	 *	数据不存在
	 */
	NO_EXISTS_DATA(777, "数据不存在"),
	/**
	 *	HTTP请求方法不支持
	 */
	ERROR_METHOD_NOT_SUPPORT(779,"HTTP请求方法不支持"),
	/**
	 *	用户认证失败
	 */
	CERTIFICATION_FAILED(780, "用户认证失败"),
	/**
	 *	找不到该用户
	 */
	NO_USER_INFO(781, "找不到该用户"),
	/**
	 *	业务操作异常
	 */
	BUSINESS_OPT_ERROR(800, "业务操作异常");

	private Integer code;

	private String message;

	private static final Set<ErrorCodeEnum> ALL = EnumSet.allOf(ErrorCodeEnum.class);

	public static ErrorCodeEnum getCodeByName(Integer code) {
		if(code==null){
			return null;
		}

		return ALL.stream()
				.filter(o->o.code.equals(code))
				.findAny().orElseThrow(()->new BusinessException("错误编码枚举类: "+code+ "未匹配到相关值！"));

	}

	public boolean is(Integer code){
		return getCode().equals(code);
	}

}
