package com.abi.task.common.utils;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;

import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;

/**
 * 时间日期工具类
 * @Author AndyTan
 * @Date 2021/1/18
 * @Param
 * @return
 */
public class IDateUtils {

    private IDateUtils() {
    }

    public static Date asDate(LocalDate localDate) {
        return Date.from(localDate.atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
    }

    public static Date asDate(LocalDateTime localDateTime) {
        return Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
    }

    public static LocalDate asLocalDate(Date date) {
        return Instant.ofEpochMilli(date.getTime()).atZone(ZoneId.systemDefault()).toLocalDate();
    }

    public static LocalDateTime asLocalDateTime(Date date) {
        return Instant.ofEpochMilli(date.getTime()).atZone(ZoneId.systemDefault()).toLocalDateTime();
    }

    public static LocalDateTime offsetMinute(LocalDateTime date,int offset){
        return asLocalDateTime(DateUtil.offsetMinute(asDate(date), offset));
    }

    public static LocalDateTime offsetDay(LocalDateTime date,int offset){
        return asLocalDateTime(DateUtil.offsetDay(asDate(date), offset));
    }

    public static LocalDateTime offsetWeek(LocalDateTime date,int offset){
        return asLocalDateTime(DateUtil.offsetWeek(asDate(date), offset));
    }

    /**
     * 日期比较
     * 如果date1<date2 返回小于0 ，=返回0，>返回大于0
     * @param date1
     * @param date2
     * @return
     */
    public static int compare(LocalDateTime date1,LocalDateTime date2){
        return DateUtil.compare(asDate(date1),asDate(date2));
    }

    /**
     * 日期是否在范围内
     * @param date
     * @param beginDate
     * @param endDate
     * @return
     */
    public static boolean isIn(LocalDateTime date, LocalDateTime beginDate, LocalDateTime endDate) {
        return DateUtil.isIn(asDate(date),asDate(beginDate),asDate(endDate));
    }

    /**
     * 去除时间的时分秒
     * @param date
     * @return
     */
    public static LocalDateTime rmTime(LocalDateTime date){
        String tempDateStr = DateUtil.format(date, "yyyy-MM-dd");
        return DateUtil.parseLocalDateTime(tempDateStr,"yyyy-MM-dd");
    }
}
