package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.service.ActiveService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 活动管理
 * </p>
 *
 * @author Andy.Tan
 */
@Slf4j
@RestController
@RequestMapping("active")
@Api(tags = "码活动管理")
public class ActiveController {

    @Autowired
    private ActiveService activeService;


    @ApiOperation("新增活动")
    @PostMapping("addActive")
    public BaseResponse addActive(@Validated @RequestBody AddActiveReq addActiveReq) throws Exception{
        //新增活动
        activeService.addActive(addActiveReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("编辑活动(待审核/审核拒绝 初次)")
    @PostMapping("updateActive")
    public BaseResponse updateActive(@Validated @RequestBody UpdateActiveReq updateActiveReq) throws Exception{
        //编辑活动
        activeService.updateActive(updateActiveReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("删除活动")
    @PostMapping("deleteActive")
    public BaseResponse deleteActive(@Validated @RequestBody DeleteActiveReq deleteActiveReq) throws Exception{
        //删除活动
        activeService.deleteActive(deleteActiveReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("修改码活动(已审核通过)")
    @PostMapping("resubmitActive")
    public BaseResponse resubmitActive(@Validated @RequestBody ResubmitActiveReq req) throws Exception{
        //编辑活动
        activeService.resubmitActive(req);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("分页查询活动")
    @GetMapping("listActive")
    public BaseResponse<ListActiveRes> listActive(@Validated ListActiveReq listActiveReq) throws Exception{
        //分页查询活动
        ListActiveRes result = activeService.listActive(listActiveReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("查询活动明细(编辑审核页面)")
    @GetMapping("getActive")
    public BaseResponse<GetActiveRes> getActive(@Validated GetActiveReq getActiveReq) throws Exception{
        //查询活动明细
        GetActiveRes  result = activeService.getActive(getActiveReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("查询活动明细(重新提交申请单审核页面)")
    @GetMapping("getActiveModifyApplyRes")
    public BaseResponse<GetActiveModifyApplyRes> getActiveModifyApplyRes(@Validated GetActiveModifyApplyReq req) throws Exception{
        //查询活动明细
        GetActiveModifyApplyRes  result = activeService.getActiveModifyApplyRes(req);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("活动启用")
    @PostMapping("enableActive")
    public BaseResponse enableActive(@Validated @RequestBody EnableActiveReq enableActiveReq) throws Exception{
        //活动启用
        activeService.enableActive(enableActiveReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("活动禁用")
    @PostMapping("disableActive")
    public BaseResponse disableActive(@Validated @RequestBody DisableActiveReq disableActiveReq) throws Exception{
        //活动禁用
        activeService.disableActive(disableActiveReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("审核通过")
    @PostMapping("/passActive")
    public BaseResponse passActive(@Validated @RequestBody PassActiveReq passActiveReq) {
        activeService.passActive(passActiveReq);
        return BaseResponse.create();
    }

    @ApiOperation("审核驳回")
    @PostMapping("/refuseActive")
    public BaseResponse refuseActive(@Validated @RequestBody RefuseActiveReq refuseActiveReq) {
        activeService.refuseActive(refuseActiveReq);
        return BaseResponse.create();
    }

    @ApiOperation("审核通过活动申请单")
    @PostMapping("/passActiveModifyApply")
    public BaseResponse passActiveModifyApply(@Validated @RequestBody PassActiveModifyApplyReq req) {
        activeService.passActiveModifyApply(req);
        return BaseResponse.create();
    }

    @ApiOperation("审核驳回活动申请单")
    @PostMapping("/refuseActiveModifyApply")
    public BaseResponse refuseActiveModifyApply(@Validated @RequestBody RefuseActiveModifyApplyReq req) {
        activeService.refuseActiveModifyApply(req);
        return BaseResponse.create();
    }

}








