package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.annotation.PassToken;
import com.abi.qms.platform.dto.req.BatchImportBrandReq;
import com.abi.qms.platform.dto.req.BatchImportCoverFactoryReq;
import com.abi.qms.platform.dto.req.BatchImportMaterialReq;
import com.abi.qms.platform.dto.req.BatchImportOrganizationReq;
import com.abi.qms.platform.dto.req.BatchImportPackageSapOrderItemReq;
import com.abi.qms.platform.dto.req.BatchImportPackageSapOrderReq;
import com.abi.qms.platform.dto.req.BatchImportSapBeerFactoryReq;
import com.abi.qms.platform.dto.req.BatchImportSkuReq;
import com.abi.qms.platform.service.BaseSapBeerFactoryService;
import com.abi.qms.platform.service.BaseSapBrandService;
import com.abi.qms.platform.service.BaseSapCoverFactoryService;
import com.abi.qms.platform.service.BaseSapMaterialService;
import com.abi.qms.platform.service.BaseSapOrganizationService;
import com.abi.qms.platform.service.BaseSapSkuService;
import com.abi.qms.platform.service.QrPackageSapOrderItemService;
import com.abi.qms.platform.service.QrPackageSapOrderService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author: fangxinjian
 * @date: 2021/06/01 15:57
 * @description:
 */
@Slf4j
@RestController
@RequestMapping("sap")
@Api(tags = "sap导入数据")
public class BaseSapImportController {

    @Autowired
    private BaseSapOrganizationService baseSapOrganizationService;

    @Autowired
    private BaseSapSkuService baseSapSkuService;

    @Autowired
    private BaseSapBrandService baseSapBrandService;

    @Autowired
    private BaseSapMaterialService baseSapMaterialService;

    @Autowired
    private BaseSapCoverFactoryService baseSapCoverFactoryService;

    @Autowired
    private BaseSapBeerFactoryService baseSapBeerFactoryService;

    @Autowired
    private QrPackageSapOrderService qrPackageSapOrderService;

    @Autowired
    private QrPackageSapOrderItemService qrPackageSapOrderItemService;


    @ApiOperation("批量导入组织架构数据")
    @PostMapping("batchImportOrganization")
    @PassToken
    public BaseResponse batchImportOrganization(@RequestBody BatchImportOrganizationReq batchImportOrganizationReq) {

        baseSapOrganizationService.batchImportOrganization(batchImportOrganizationReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("批量修改组织架构数据")
    @PostMapping("batchUpdateOrganization")
    @PassToken
    public BaseResponse batchUpdateOrganization(@RequestBody BatchImportOrganizationReq batchImportOrganizationReq) {

        baseSapOrganizationService.batchUpdateOrganization(batchImportOrganizationReq);
        //包装出参
        return BaseResponse.create();
    }


    @ApiOperation("批量导入SKU数据")
    @PostMapping("batchImportSku")
    @PassToken
    public BaseResponse batchImportSku(@RequestBody BatchImportSkuReq batchImportSkuReq) {

        baseSapSkuService.batchImportSku(batchImportSkuReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("批量修改SKU数据")
    @PostMapping("batchUpdateSku")
    @PassToken
    public BaseResponse batchUpdateSku(@RequestBody BatchImportSkuReq batchImportSkuReq) {

        baseSapSkuService.batchUpdateSku(batchImportSkuReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("批量导入品牌数据")
    @PostMapping("batchImportBrand")
    @PassToken
    public BaseResponse batchImportBrand(@RequestBody BatchImportBrandReq batchImportBrandReq) {

        baseSapBrandService.batchImportBrand(batchImportBrandReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("批量修改品牌数据")
    @PostMapping("batchUpdateBrand")
    @PassToken
    public BaseResponse batchUpdateBrand(@RequestBody BatchImportBrandReq batchImportBrandReq) {

        baseSapBrandService.batchUpdateBrand(batchImportBrandReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("批量导入Sap物料数据")
    @PostMapping("batchImportMaterial")
    @PassToken
    public BaseResponse batchImportMaterial(@RequestBody BatchImportMaterialReq batchImportMaterialReq) {

        baseSapMaterialService.batchImportMaterial(batchImportMaterialReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("批量修改Sap物料数据")
    @PostMapping("batchUpdateMaterial")
    @PassToken
    public BaseResponse batchUpdateMaterial(@RequestBody BatchImportMaterialReq batchImportMaterialReq) {

        baseSapMaterialService.batchUpdateMaterial(batchImportMaterialReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("批量导入Sap供应商数据")
    @PostMapping("batchImportCoverFactory")
    @PassToken
    public BaseResponse batchImportCoverFactory(@RequestBody BatchImportCoverFactoryReq batchImportCoverFactoryReq) {

        baseSapCoverFactoryService.batchImportCoverFactory(batchImportCoverFactoryReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("批量修改Sap供应商数据")
    @PostMapping("batchUpdateCoverFactory")
    @PassToken
    public BaseResponse batchUpdateCoverFactory(@RequestBody BatchImportCoverFactoryReq batchImportCoverFactoryReq) {

        baseSapCoverFactoryService.batchUpdateCoverFactory(batchImportCoverFactoryReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("批量导入Sap啤酒厂数据")
    @PostMapping("batchImportSapBeerFactory")
    @PassToken
    public BaseResponse batchImportSapBeerFactory(@RequestBody BatchImportSapBeerFactoryReq batchImportSapBeerFactoryReq) {

        baseSapBeerFactoryService.batchImportSapBeerFactory(batchImportSapBeerFactoryReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("批量修改Sap啤酒厂数据")
    @PostMapping("batchUpdateSapBeerFactory")
    @PassToken
    public BaseResponse batchUpdateSapBeerFactory(@RequestBody BatchImportSapBeerFactoryReq batchImportSapBeerFactoryReq) {

        baseSapBeerFactoryService.batchUpdateSapBeerFactory(batchImportSapBeerFactoryReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("批量导入Sap码包订单数据")
    @PostMapping("batchImportPackageSapOrder")
    @PassToken
    public BaseResponse batchImportPackageSapOrder(@RequestBody BatchImportPackageSapOrderReq batchImportPackageSapOrderReq) {

        qrPackageSapOrderService.batchImportPackageSapOrder(batchImportPackageSapOrderReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("批量修改Sap码包订单数据")
    @PostMapping("batchUpdatePackageSapOrder")
    @PassToken
    public BaseResponse batchUpdatePackageSapOrder(@RequestBody BatchImportPackageSapOrderReq batchImportPackageSapOrderReq) {

        qrPackageSapOrderService.batchUpdatePackageSapOrder(batchImportPackageSapOrderReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("批量导入Sap物料订单明细数据")
    @PostMapping("batchImportPackageSapOrderItem")
    @PassToken
    public BaseResponse batchImportPackageSapOrderItem(@RequestBody BatchImportPackageSapOrderItemReq batchImportPackageSapOrderItemReq) {

        qrPackageSapOrderItemService.batchImportPackageSapOrderItem(batchImportPackageSapOrderItemReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("批量修改SAP物料订单明细数据")
    @PostMapping("batchUpdatePackageSapOrderItem")
    @PassToken
    public BaseResponse batchUpdatePackageSapOrderItem(@RequestBody BatchImportPackageSapOrderItemReq batchImportPackageSapOrderItemReq) {

        qrPackageSapOrderItemService.batchUpdatePackageSapOrderItem(batchImportPackageSapOrderItemReq);
        //包装出参
        return BaseResponse.create();
    }


}
