package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.service.SapBaseQueryService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
 * sap基础数据查询
 * </p>
 *
 * @author Andy.Tan
 */
@Slf4j
@RestController
@RequestMapping("/sap/query")
@Api(tags = "sap基础数据查询")
public class BaseSapQueryController {

    @Autowired
    private SapBaseQueryService sapBaseQueryService;

    @ApiOperation("查询SAP物料列表")
    @GetMapping("listSapMaterial")
    public BaseResponse<ListSapMaterialRes> listSapMaterial(@Validated ListSapMaterialReq listSapMaterialReq) throws Exception{
        //查询SAP物料列表
        ListSapMaterialRes  result = sapBaseQueryService.listSapMaterial(listSapMaterialReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("查询SAP包材场列表")
    @GetMapping("listSapCoverFactory")
    public BaseResponse<ListSapCoverFactoryRes> listSapCoverFactory(@Validated ListSapCoverFactoryReq listSapCoverFactoryReq) throws Exception{
        //查询SAP包材场列表
        ListSapCoverFactoryRes  result = sapBaseQueryService.listSapCoverFactory(listSapCoverFactoryReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("查询SAP啤酒厂列表")
    @GetMapping("listSapBeerFactory")
    public BaseResponse<ListSapBeerFactoryRes> listSapBeerFactory(@Validated ListSapBeerFactoryReq listSapBeerFactoryReq) throws Exception{
        //查询SAP啤酒厂列表
        ListSapBeerFactoryRes  result = sapBaseQueryService.listSapBeerFactory(listSapBeerFactoryReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("查询SAP订单列表")
    @GetMapping("listSapCodePackageOrder")
    public BaseResponse<ListSapCodePackageOrderRes> listSapCodePackageOrder(@Validated ListSapCodePackageOrderReq listSapCodePackageOrderReq) throws Exception{
        //查询SAP订单列表
        ListSapCodePackageOrderRes  result = sapBaseQueryService.listSapCodePackageOrder(listSapCodePackageOrderReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("查询SAP订单关联的QMS物料列表")
    @GetMapping("listMaterialBySapOrder")
    public BaseResponse<ListMaterialBySapOrderRes> listMaterialBySapOrder(@Validated ListMaterialBySapOrderReq listMaterialBySapOrderReq) throws Exception{
        //查询SAP订单关联的QMS物料列表
        ListMaterialBySapOrderRes result = sapBaseQueryService.listMaterialBySapOrder(listMaterialBySapOrderReq);
        //包装出参
        return BaseResponse.create(result);
    }



}








