package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.annotation.PassToken;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.GetLoginUserInfoRes;
import com.abi.qms.platform.dto.res.LoginRes;
import com.abi.qms.platform.service.LoginService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 登录
 * </p>
 *
 * @author Andy.Tan
 */
@Slf4j
@RestController
@RequestMapping("login")
@Api(tags = "登录")
public class LoginController {

    @Autowired
    private LoginService loginService;

    @PostMapping("login")
    @ApiOperation("登录QMS")
    @PassToken
    public BaseResponse<LoginRes> loginConsole(@Validated @RequestBody LoginReq req) {
        LoginRes login = loginService.loginConsole(req);
        return BaseResponse.create(login);
    }

    @ApiOperation("重置密码")
    @PostMapping("resetPassword")
    public BaseResponse resetPassword(@Validated @RequestBody ResetPasswordReq req) throws Exception {
        loginService.resetPassword(req);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("修改密码")
    @PostMapping("updatePassword")
    public BaseResponse updatePassword(@Validated @RequestBody UpdatePasswordReq req) throws Exception {
        loginService.updatePassword(req);
        return BaseResponse.create();
    }

    @ApiOperation("修改登录用户邮箱")
    @PostMapping("/updateLoginUserEmail")
    public BaseResponse updateLoginUserEmail(@Validated @RequestBody UpdateLoginUserEmailReq updateLoginUserEmailReq) throws Exception{
        //修改登录用户邮箱
        loginService.updateLoginUserEmail(updateLoginUserEmailReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("查询登录用户信息")
    @GetMapping("/getLoginUserInfo")
    public BaseResponse<GetLoginUserInfoRes> getLoginUserInfo(@Validated GetLoginUserInfoReq getLoginUserInfoReq) throws Exception{
        //查询登录用户信息
        GetLoginUserInfoRes result = loginService.getLoginUserInfo(getLoginUserInfoReq);
        //包装出参
        return BaseResponse.create(result);
    }

}








