package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dto.req.QrBoxCodeFormatDetailReq;
import com.abi.qms.platform.dto.req.DeleteBoxCodeFormatreq;
import com.abi.qms.platform.dto.req.DisableBoxCodeFormatReq;
import com.abi.qms.platform.dto.req.EnableBoxCodeFormatReq;
import com.abi.qms.platform.dto.req.ListBoxCodeFormatReq;
import com.abi.qms.platform.dto.req.SaveBoxCodeFormatReq;
import com.abi.qms.platform.dto.res.QrBoxCodeFormatDetailRes;
import com.abi.qms.platform.dto.res.ListBoxCodeFormatRes;
import com.abi.qms.platform.service.QrBoxCodeFormatService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author: fangxinjian
 * @date: 2021/04/20 09:45
 * @description: 箱码幅面管理
 */

@Slf4j
@RestController
@RequestMapping("boxCodeFormat")
@Api(tags = "箱码幅面管理")
public class QrBoxCodeFormatController {

    @Autowired
    private QrBoxCodeFormatService qrBoxCodeFormatService;


    @ApiOperation("箱码幅面新增/修改")
    @PostMapping("saveBoxCodeFormat")
    public BaseResponse saveBoxCodeFormat(@Validated @RequestBody SaveBoxCodeFormatReq saveBoxCodeFormatReq) {
        //箱码幅面新增修改
        qrBoxCodeFormatService.saveBoxCodeFormat(saveBoxCodeFormatReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("箱码幅面查询分页")
    @GetMapping("listBoxCodeFormat")
    public BaseResponse<ListBoxCodeFormatRes> listBoxCodeFormat(@Validated ListBoxCodeFormatReq listBoxCodeFormatReq) {
        //箱码幅面查询分页
        ListBoxCodeFormatRes result = qrBoxCodeFormatService.listBoxCodeFormat(listBoxCodeFormatReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("箱码幅面查询详情")
    @GetMapping("getBoxCodeFormatDetail")
    public BaseResponse<QrBoxCodeFormatDetailRes> getBoxCodeFormatDetail(@Validated QrBoxCodeFormatDetailReq qrBoxCodeFormatDetailReq) {
        //箱码幅面查询详情
        QrBoxCodeFormatDetailRes result = qrBoxCodeFormatService.getBoxCodeFormatDetail(qrBoxCodeFormatDetailReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("箱码幅面启用")
    @PostMapping("enableBoxCodeFormat")
    public BaseResponse enableBoxCodeFormat(@Validated @RequestBody EnableBoxCodeFormatReq enableBoxCodeFormatReq) throws Exception {
        //箱码幅面启用
        qrBoxCodeFormatService.enableBoxCodeFormat(enableBoxCodeFormatReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("箱码幅面禁用")
    @PostMapping("disableBoxCodeFormat")
    public BaseResponse disableBoxCodeFormat(@Validated @RequestBody DisableBoxCodeFormatReq disableBoxCodeFormatReq) throws Exception {
        //箱码幅面禁用
        qrBoxCodeFormatService.disableBoxCodeFormat(disableBoxCodeFormatReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("删除箱码幅面")
    @PostMapping("deleteBoxCodeFormat")
    public BaseResponse deleteBoxCodeFormat(@Validated @RequestBody DeleteBoxCodeFormatreq deleteBoxCodeFormatreq) throws Exception {
        //删除箱码幅面
        qrBoxCodeFormatService.deleteBoxCodeFormat(deleteBoxCodeFormatreq);
        //包装出参
        return BaseResponse.create();
    }

}
