package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dto.req.DeleteQrFormatReq;
import com.abi.qms.platform.dto.req.DisableQrFormatReq;
import com.abi.qms.platform.dto.req.EnableQrFormatReq;
import com.abi.qms.platform.dto.req.GetQrFormatDetailReq;
import com.abi.qms.platform.dto.req.ListQrFormatReq;
import com.abi.qms.platform.dto.req.SaveQrFormatReq;
import com.abi.qms.platform.dto.res.GetQrFormatDetailRes;
import com.abi.qms.platform.dto.res.ListFormatSelectRes;
import com.abi.qms.platform.dto.res.ListQrFormatRes;
import com.abi.qms.platform.service.QrFormatService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 码格式 Controller
 *
 * @author WeiganCai
 * @date 2021-04-19
 */
@Slf4j
@RestController
@Api(tags = "码格式管理")
@RequestMapping("/qrFormat")
public class QrFormatController {
    @Autowired
    private QrFormatService qrFormatService;

    @ApiOperation("码格式新增修改")
    @PostMapping("/saveQrFormat")
    public BaseResponse saveQrFormat(@Validated @RequestBody SaveQrFormatReq saveQrFormatReq) {
        qrFormatService.saveQrFormat(saveQrFormatReq);
        return BaseResponse.create();
    }

    @ApiOperation("码格式查询分页")
    @GetMapping("/listQrFormat")
    public BaseResponse<ListQrFormatRes> listQrFormat(@Validated ListQrFormatReq listQrFormatReq) {
        ListQrFormatRes result = qrFormatService.listQrFormat(listQrFormatReq);
        return BaseResponse.create(result);
    }

    @ApiOperation("码格式查询详情")
    @GetMapping("/getQrFormatDetail")
    public BaseResponse<GetQrFormatDetailRes> getQrFormatDetail(@Validated GetQrFormatDetailReq getQrFormatDetailReq) {
        GetQrFormatDetailRes result = qrFormatService.getQrFormatDetail(getQrFormatDetailReq);
        return BaseResponse.create(result);
    }

    @ApiOperation("码格式启用")
    @PostMapping("/enableQrFormat")
    public BaseResponse enableQrFormat(@Validated @RequestBody EnableQrFormatReq enableQrFormatReq) {
        qrFormatService.enableQrFormat(enableQrFormatReq);
        return BaseResponse.create();
    }

    @ApiOperation("码格式禁用")
    @PostMapping("/disableQrFormat")
    public BaseResponse disableQrFormat(@Validated @RequestBody DisableQrFormatReq disableQrFormatReq) {
        qrFormatService.disableQrFormat(disableQrFormatReq);
        return BaseResponse.create();
    }

    @ApiOperation("删除码格式")
    @PostMapping("/deleteQrFormat")
    public BaseResponse deleteQrFormat(@Validated @RequestBody DeleteQrFormatReq deleteQrFormatReq) {
        qrFormatService.deleteQrFormat(deleteQrFormatReq);
        return BaseResponse.create();
    }

    @ApiOperation("码格式下拉列表")
    @GetMapping("/listFormatSelect")
    public BaseResponse<ListFormatSelectRes> listFormatSelect() {

        ListFormatSelectRes res = qrFormatService.listFormatSelect();
        return BaseResponse.create(res);
    }

}
