package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dto.req.DeleteQrRepertoryReq;
import com.abi.qms.platform.dto.req.DisableQrRepertoryReq;
import com.abi.qms.platform.dto.req.EnableQrRepertoryReq;
import com.abi.qms.platform.dto.req.GetQrRepertoryDetailReq;
import com.abi.qms.platform.dto.req.ListQrRepertoryReq;
import com.abi.qms.platform.dto.req.ListQrRepertorySelectDetailReq;
import com.abi.qms.platform.dto.req.SaveQrRepertoryReq;
import com.abi.qms.platform.dto.res.GetQrRepertoryDetailRes;
import com.abi.qms.platform.dto.res.ListQrRepertoryRes;
import com.abi.qms.platform.dto.res.ListQrRepertorySelectDetailRes;
import com.abi.qms.platform.dto.res.ListQrRepertorySelectRes;
import com.abi.qms.platform.service.QrRepertoryService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 码库 Controller
 *
 * @author WeiganCai
 * @date 2021-04-19
 */
@Slf4j
@RestController
@Api(tags = "码库管理")
@RequestMapping("/qrRepertory")
public class QrRepertoryController {
    @Autowired
    private QrRepertoryService qrRepertoryService;

    @ApiOperation("码库新增修改")
    @PostMapping("/saveQrRepertory")
    public BaseResponse saveQrRepertory(@Validated @RequestBody SaveQrRepertoryReq saveQrRepertoryReq) {
        qrRepertoryService.saveQrRepertory(saveQrRepertoryReq);
        return BaseResponse.create();
    }

    @ApiOperation("码库查询分页")
    @GetMapping("/listQrRepertory")
    public BaseResponse<ListQrRepertoryRes> listQrRepertory(@Validated ListQrRepertoryReq listQrRepertoryReq) {
        ListQrRepertoryRes result = qrRepertoryService.listQrRepertory(listQrRepertoryReq);
        return BaseResponse.create(result);
    }

    @ApiOperation("码库查询详情")
    @GetMapping("/getQrRepertoryDetail")
    public BaseResponse<GetQrRepertoryDetailRes> getQrRepertoryDetail(@Validated GetQrRepertoryDetailReq getQrRepertoryDetailReq) {
        GetQrRepertoryDetailRes result = qrRepertoryService.getQrRepertoryDetail(getQrRepertoryDetailReq);
        return BaseResponse.create(result);
    }

    @ApiOperation("码库启用")
    @PostMapping("/enableQrRepertory")
    public BaseResponse enableQrRepertory(@Validated @RequestBody EnableQrRepertoryReq enableQrRepertoryReq) {
        qrRepertoryService.enableQrRepertory(enableQrRepertoryReq);
        return BaseResponse.create();
    }

    @ApiOperation("码库禁用")
    @PostMapping("/disableQrRepertory")
    public BaseResponse disableQrRepertory(@Validated @RequestBody DisableQrRepertoryReq disableQrRepertoryReq) {
        qrRepertoryService.disableQrRepertory(disableQrRepertoryReq);
        return BaseResponse.create();
    }

    @ApiOperation("删除码库")
    @PostMapping("/deleteQrRepertory")
    public BaseResponse deleteQrRepertory(@RequestBody DeleteQrRepertoryReq deleteQrRepertoryReq) {
        qrRepertoryService.deleteQrRepertory(deleteQrRepertoryReq);
        return BaseResponse.create();
    }

    @ApiOperation("查询码库下拉列表")
    @GetMapping("/listQrRepertorySelect")
    public BaseResponse<ListQrRepertorySelectRes> listQrRepertorySelectRes() {

        ListQrRepertorySelectRes res = qrRepertoryService.listQrRepertorySelectRes();
        return BaseResponse.create(res);
    }

    @ApiOperation("查询码库下拉详情")
    @GetMapping("/listQrRepertorySelectDetail")
    public BaseResponse<ListQrRepertorySelectDetailRes> listQrRepertorySelectDetail(@Validated ListQrRepertorySelectDetailReq req) {

        ListQrRepertorySelectDetailRes res = qrRepertoryService.listQrRepertorySelectDetail(req);
        return BaseResponse.create(res);
    }

}
