package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.service.RoleManagerService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 角色管理
 * </p>
 *
 * @author Andy.Tan
 */
@Slf4j
@RestController
@Api(tags = "角色管理")
@RequestMapping("role")
public class UserRoleController {

    @Autowired
    private RoleManagerService roleManagerService;

    @ApiOperation("获取选择的上级角色对应的菜单/操作权限树")
    @GetMapping("listResourceTreeBySelectRoleCode")
    public BaseResponse<ListResourceTreeBySelectRoleCodeRes> listResourceTreeBySelectRoleCode(@Validated ListResourceTreeBySelectRoleCodeReq listResourceTreeBySelectRoleCodeReq) throws Exception{
        //获取登录角色下级的角色树
        ListResourceTreeBySelectRoleCodeRes result = roleManagerService.listResourceTreeBySelectRoleCode(listResourceTreeBySelectRoleCodeReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("获取登录用户的页面以及按钮权限")
    @GetMapping("listLoginUserResource")
    public BaseResponse<ListLoginUserResourceRes> listLoginUserResource() throws Exception{
        //获取登录用户需要的页面以及按钮权限
        ListLoginUserResourceRes result = roleManagerService.listLoginUserResource();
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("创建角色")
    @PostMapping("createRole")
    public BaseResponse createRole(@Validated @RequestBody CreateRoleReq createRoleReq) throws Exception{
        //创建角色
        roleManagerService.createRole(createRoleReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("修改角色")
    @PostMapping("updateRole")
    public BaseResponse updateRole(@Validated @RequestBody UpdateRoleReq updateRoleReq) throws Exception{
        //修改角色
        roleManagerService.updateRole(updateRoleReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("查询单个角色信息")
    @GetMapping("getRoleInfo")
    public BaseResponse<GetRoleInfoRes> getRoleInfo(@Validated GetRoleInfoReq getRoleInfoReq) throws Exception{
        //查询单个角色信息
        GetRoleInfoRes result = roleManagerService.getRoleInfo(getRoleInfoReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("查询角色列表")
    @GetMapping("listRole")
    public BaseResponse<ListRoleRes> listRole(@Validated ListRoleReq listRoleReq) throws Exception{
        //查询角色列表
        ListRoleRes result = roleManagerService.listRole(listRoleReq);
        //包装出参
        return BaseResponse.create(result);
    }

    @ApiOperation("启用角色")
    @PostMapping("enableRole")
    public BaseResponse enableRole(@Validated @RequestBody EnableRoleReq enableRoleReq) throws Exception{
        //启用角色
        roleManagerService.enableRole(enableRoleReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("禁用角色")
    @PostMapping("disableRole")
    public BaseResponse disableRole(@Validated @RequestBody DisableRoleReq disableRoleReq) throws Exception{
        //禁用角色
        roleManagerService.disableRole(disableRoleReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("获取登录角色下的资源列表（可列可树形） 这个其实感觉现在不需要了")
    @GetMapping("listResourceTreeByRoleCode")
    public BaseResponse<ListResourceTreeByRoleCodeRes> listResourceTreeByRoleCode(@Validated ListResourceTreeByRoleCodeReq listResourceTreeByRoleCodeReq) throws Exception{
        //获取登录角色下级的角色树
        ListResourceTreeByRoleCodeRes result = roleManagerService.listResourceTreeByRoleCode(listResourceTreeByRoleCodeReq);
        //包装出参
        return BaseResponse.create(result);
    }

}








