package com.abi.qms.platform.controller.wx;

import com.abi.qms.platform.annotation.PassToken;
import com.abi.qms.platform.dto.req.LoginReq;
import com.abi.qms.platform.dto.req.ResetPasswordReq;
import com.abi.qms.platform.dto.req.UpdatePasswordReq;
import com.abi.qms.platform.dto.res.LoginActivateRes;
import com.abi.qms.platform.dto.res.LoginRes;
import com.abi.qms.platform.service.LoginService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
 * 登录
 * </p>
 *
 * @author Andy.Tan
 */
@Slf4j
@RestController
@RequestMapping("wx/login")
@Api(tags = "登录")
public class WxLoginController {

    @Autowired
    private LoginService loginService;

    @PostMapping("loginActivate")
    @ApiOperation("登录激活小程序")
    @PassToken
    public BaseResponse<LoginActivateRes> loginActivate(@Validated @RequestBody LoginReq req) {
        LoginActivateRes login = loginService.loginActivate(req);
        return BaseResponse.create(login);
    }

    @PostMapping("updatePassword")
    @ApiOperation("修改密码")
    public BaseResponse<LoginActivateRes> updatePassword(@Validated @RequestBody UpdatePasswordReq req) {
        loginService.updatePassword(req);
        return BaseResponse.create();
    }
}








