package com.abi.qms.platform.controller.wx;

import com.abi.qms.platform.dto.req.ActivateDetailReq;
import com.abi.qms.platform.dto.req.ActivateNowReq;
import com.abi.qms.platform.dto.req.ActivationRecordReq;
import com.abi.qms.platform.dto.req.QueryWxQrBoxMappingDetailsWxReq;
import com.abi.qms.platform.dto.req.ReplenishActivationReq;
import com.abi.qms.platform.dto.res.ActivateDetailRes;
import com.abi.qms.platform.dto.res.ActivationRecordRes;
import com.abi.qms.platform.dto.res.QueryWxQrBoxMappingDetailsWxRes;
import com.abi.qms.platform.service.QrBoxMappingService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author lu
 * @date 2021年06月09日 10:10
 */
@Slf4j
@RestController
@RequestMapping("/wxQrBoxMapping")
@Api(tags = "微信箱码管理")
public class WxQrBoxMappingController {

    @Autowired
    private QrBoxMappingService qrBoxMappingService;

    /**
     * 微信 - 箱码/条码详情
     *
     * @param queryWxQrBoxMappingDetailsWxReq
     * @return com.abi.task.common.api.base.BaseResponse<com.abi.qms.platform.dto.res.QueryWxQrBoxMappingDetailsWxRes>
     * @author lu
     * @date 2021/6/9 11:24
     */
    @ApiOperation("激活详情")
    @PostMapping("queryWxQrBoxMappingDetails")
    public BaseResponse<QueryWxQrBoxMappingDetailsWxRes> queryWxQrBoxMappingDetails(@Validated @RequestBody QueryWxQrBoxMappingDetailsWxReq queryWxQrBoxMappingDetailsWxReq) {
        QueryWxQrBoxMappingDetailsWxRes res = qrBoxMappingService.queryWxQrBoxMappingDetails(queryWxQrBoxMappingDetailsWxReq);
        return BaseResponse.create(res);
    }

    @ApiOperation("补充激活")
    @PostMapping("replenishActivation")
    public BaseResponse replenishActivation(@RequestBody ReplenishActivationReq replenishActivationReq) {
        //补充激活
        qrBoxMappingService.replenishActivation(replenishActivationReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("激活记录")
    @GetMapping("getActivationRecord")
    public BaseResponse<ActivationRecordRes> getActivationRecord(@Validated ActivationRecordReq activationRecordReq) {
        //补充激活
        ActivationRecordRes res = qrBoxMappingService.getActivationRecord(activationRecordReq);
        //包装出参
        return BaseResponse.create(res);
    }

    @ApiOperation("立即激活")
    @PostMapping("activateNow")
    public BaseResponse activateNow(@RequestBody @Validated ActivateNowReq activateNowReq) {
        //立即激活
        qrBoxMappingService.activateNow(activateNowReq);
        //包装出参
        return BaseResponse.create();
    }

    @ApiOperation("箱码/条码激活详情")
    @GetMapping("getActivateDetail")
    public BaseResponse<ActivateDetailRes> getActivateDetail(@Validated ActivateDetailReq activateDetailReq) {
        //立即激活
        ActivateDetailRes res = qrBoxMappingService.getActivateDetail(activateDetailReq);
        //包装出参
        return BaseResponse.create(res);
    }

}
