package com.abi.qms.platform.dto.req;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.ToString;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.List;

/**
 * 码包新增修改入参
 *
 * @author WeiganCai
 * @date: 2021-04-28
 */
@Data
@ApiModel
public class SaveQrPackageReq implements Serializable {

	@ApiModelProperty("码包id")
	private Long id;

	@NotNull(message = "订单类型为空")
	@ApiModelProperty(value = "订单类型: 1-SAP订单 2-预订单")
	private Integer orderType;

	@Size(max = 20, message = "SAP单号不能超过20字")
	@ApiModelProperty(value = "SAP单号")
	private String sapOrderNo;

	@Size(max = 20, message = "预订单号不能超过20字")
	@ApiModelProperty(value = "预订单号")
	private String bookingOrder;

	@ApiModelProperty(value = "物料id")
	private Long materialId;

	@ApiModelProperty(value = "物料号")
	private String materialCode;

	@NotNull(message = "包材厂为空")
	@ApiModelProperty(value = "包材厂id")
	private Long factoryCoverId;

	@ApiModelProperty(value = "二级包材厂id")
	private Long factoryCoverLevelTwoId;

	@ApiModelProperty(value = "啤酒厂id")
	private Long factoryBeerId;

	@ApiModelProperty("箱码幅面id")
	private Long boxCodeFormatId;

	@ApiModelProperty(value = "备用码包")
	private Integer standbyRatio;

	@NotNull(message = "码数量为空")
	@ApiModelProperty("码数量")
	private Long qrNumber;

	@Size(max = 50, message = "申请原因超过50字")
	@ApiModelProperty(value = "申请原因")
	private String reasonApply;

	@ApiModelProperty(value = "二维码类型 1-箱码 2-盖码 前端不用传")
	private Integer qrType;

	@ApiModelProperty(value = "批次列表")
	private List<PackageBatchBean> packageBatchBeanList;

	/**
	 * 批次号bean
	 */
	@Data
	@NoArgsConstructor
	@AllArgsConstructor
	@ToString
	@ApiModel(value = "SaveQrPackageReq_PackageBatchBean")
	public static class PackageBatchBean implements Serializable {

		@ApiModelProperty("批次号id")
		private Long id;

		@NotEmpty(message = "批次号为空")
		@ApiModelProperty(value = "批次号")
		private String batchNumber;

		@NotNull(message = "批次内码数量为空")
		@ApiModelProperty(value = "批次内码数量")
		private Long batchQrNumber;
	}
}
