package com.abi.qms.platform.infrastructure.aop;

import cn.hutool.json.JSONUtil;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

/**
 * @Author: yuliang
 * @Date: 2020/5/23 0023 16:12
 */
@Component
@Aspect
@Slf4j
public class ControllerMethodParamLog {
    @Pointcut("execution(* com.abi..controller..*(..))")
    public void aspect() {

    }

    @Around("aspect()")
    public Object before(ProceedingJoinPoint point) throws Throwable {
        //获取方法
        ServletRequestAttributes sra = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = sra.getRequest();
        //访问路径
        String requestPath = request.getRequestURI();
        String className = point.getSignature().getDeclaringType().getSimpleName();
        String methodName = point.getSignature().getName();
        long startTime = System.currentTimeMillis();
        MethodSignature signature = (MethodSignature) point.getSignature();
        String urlDes = signature.getMethod().getAnnotation(ApiOperation.class).value();
        //请求路径,类名,方法名
        log.info(" ");
        log.info("=====>请求路径:" + requestPath);
        log.info("=====>Controller方法:" + className + "." + methodName + "()");
        log.info("=====>方法描述:" + urlDes);
        log.info("=====>入参:" + getArgs(point));
        log.info(" ");
        Object proceed = point.proceed();
        log.info(" ");
        log.info("=====>Controller结束:" + className + "." + methodName + "()");
        log.info("=====>出参: " + JSONUtil.toJsonStr(proceed));
        log.info("=====>耗时 : " + (System.currentTimeMillis() - startTime));
        log.info(" ");
        return proceed;
    }

    private String getArgs(JoinPoint point) {
        try {
            String[] parameterNames = ((MethodSignature) point.getSignature()).getParameterNames();
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < parameterNames.length; i++) {
                String args = String.valueOf(point.getArgs()[i]);
                sb.append(parameterNames[i] + ":" + args + ";");
            }
            return sb.toString();
        } catch (Exception e) {
            log.info("拼接入参异常", e);
            return "";
        }
    }
}
