package com.abi.qms.platform.infrastructure.config.mybatis;

import com.abi.qms.platform.infrastructure.util.TokenUtil;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.baomidou.mybatisplus.annotation.DbType;
import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.baomidou.mybatisplus.extension.plugins.MybatisPlusInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.PaginationInnerInterceptor;
import org.apache.ibatis.reflection.MetaObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.time.LocalDateTime;

/**
 * mybatisPlus配置类
 *
 * @Author AndyTan
 * @Date 2021/1/18
 */
@Configuration
public class MybatisPlusConfig {

    /**
     * 新的分页插件,一缓和二缓遵循mybatis的规则,需要设置 MybatisConfiguration#useDeprecatedExecutor = false 避免缓存出现问题
     */
    @Bean
    public MybatisPlusInterceptor mybatisPlusInterceptor() {
        MybatisPlusInterceptor interceptor = new MybatisPlusInterceptor();
        interceptor.addInnerInterceptor(new PaginationInnerInterceptor(DbType.MYSQL));
        return interceptor;
    }

    /**
     * 数据库sql返回结果和时间打印
     */
    @Bean
    public ResultSqlInterceptor paginationInterceptor1() {
        return new ResultSqlInterceptor();
    }

    /**
     * 数据库sql打印
     */
    @Bean
    public SqlStatementInterceptor paginationInterceptor2() {
        return new SqlStatementInterceptor();
    }

    @Bean
    public MetaObjectHandler metaObjectHandler() {
        return new MetaObjectHandler() {

            @Override
            public void insertFill(MetaObject metaObject) {
                setFieldValByName("createTime", LocalDateTime.now(), metaObject);
                setFieldValByName("updateTime", LocalDateTime.now(), metaObject);
                //操作人
                if(TokenUtil.getUser()!=null && TokenUtil.getUser().getId()!=null){
                    setFieldValByName("updateBy", TokenUtil.getUser().getId(), metaObject);
                    setFieldValByName("createBy", TokenUtil.getUser().getId(), metaObject);
                }
            }

            @Override
            public void updateFill(MetaObject metaObject) {
                setFieldValByName("updateTime", LocalDateTime.now(), metaObject);
                //操作人
                if(TokenUtil.getUser()!=null && TokenUtil.getUser().getId()!=null){
                    setFieldValByName("updateBy", TokenUtil.getUser().getId(), metaObject);
                }
            }
        };
    }
}