package com.abi.qms.platform.infrastructure.event;

import cn.hutool.core.exceptions.ExceptionUtil;
import com.abi.qms.platform.dao.entity.QrPackage;
import com.abi.qms.platform.dao.enums.QrPackageGenerateStatusEnum;
import com.abi.qms.platform.service.GenerateCodeService;
import com.abi.qms.platform.service.QrPackageService;
import com.abi.task.common.api.exception.BusinessException;
import freemarker.template.TemplateException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.io.IOException;


@Slf4j
@Component
public class CodePackageCreateListener {
    @Autowired
    private GenerateCodeService generateCodeService;

    @Autowired
    private QrPackageService qrPackageService;


    @EventListener
    @Async
    public void onApplicationEvent(final CodePackageCreateEvent event) throws InterruptedException, IOException, TemplateException {
        log.info("CodePackageCreateListener begin");
        Long qrPackageId = (Long) event.getSource();

        try {
            // 生成码
            generateCodeService.generateQrCode(qrPackageId);
        } catch (Exception e) {
            log.error("生成码失败", e);
            // 回滚
            QrPackage qrPackage = new QrPackage();
            qrPackage.setId(qrPackageId);
            qrPackage.setGenerateStatus(QrPackageGenerateStatusEnum.GENERATE_FAIL.getCode());
            qrPackage.setGenerateErrStackInfo(e.getMessage()+" stack:"+ExceptionUtil.stacktraceToOneLineString(e));
            qrPackageService.failedRollback(qrPackage);
        }
    }

}
