package com.abi.qms.platform.infrastructure.exception;

import com.abi.task.common.api.exception.BusinessException;

import java.util.EnumSet;
import java.util.Set;

/**
 * @author Eurus
 * @date 2021/4/16 10:19
 */
public enum ErrorCodeEnum {

    SUCCESS(200, "执行成功"),
    ERROR_PARAM(615400, "参数为空或错误"),
    FAIL(615405, "执行失败"),
    USER_NOT_EXISTED(615407, "登录信息错误"),
    TOKEN_NOT_EMPTY(615408, "token不能为空"),
    TOKEN_ERROR(615409, "token错误"),
    FEIGN_ERROR_FALLBACK(615410, "feign调用失败"),
    DATE_FORMAT_ERROR(615411, "时间格式错误"),
    PASSWORD_NOT_SAME(615413, "密码不一致"),
    JSON_PARSE_ERROR(615412, "json解析异常"),
    PASSWORD_ERROR(615413, "密码错误"),
    USERNAME_DUPLICATE(615414, "该用户名已被占用，请重新输入"),
    EMPLOYEE_NO_DUPLICATE(615415, "该工号已被占用，请重新输入"),
    EMAIL_DUPLICATE(615416, "该邮箱已被占用，请重新输入"),
    USERNAME_OR_NO_EXITED(615419, "用户名或工号已存在"),
    PHONE_DUPLICATE(615420, "该手机号已被占用，请重新输入"),
    USER_ROLE_NOT_EXISTED(615421, "该用户的角色不存在"),
    TOKEN_TIME_OUT(401,"当前登录已被剔出，请重新登录");


    private Integer code;
    private String message;
    private static final Set<ErrorCodeEnum> ALL = EnumSet.allOf(ErrorCodeEnum.class);

    public ErrorCodeEnum getByCode(Integer code) {
        return (ErrorCodeEnum)ALL.stream().filter((o) -> {
            return o.code.equals(code);
        }).findAny().orElseThrow(() -> {
            return new BusinessException("错误编码枚举类: " + code + "未匹配到相关值！");
        });
    }

    public Integer getCode() {
        return this.code;
    }

    public String getMessage() {
        return this.message;
    }

    private ErrorCodeEnum(final Integer code, final String message) {
        this.code = code;
        this.message = message;
    }
}
