package com.abi.qms.platform.infrastructure.mq;

import cn.hutool.json.JSONUtil;
import com.abi.qms.platform.dao.tablestore.entity.QrCode;
import com.abi.task.common.tablestore.TableStorePlusUtils;
import com.rabbitmq.client.Channel;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AcknowledgeMode;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.listener.api.ChannelAwareMessageListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @className: com.abi.qms.platform.infrastructure.mq-> TableStoreBatchInsertConsumer
 * @description: tableStore batchInsert
 * @author: Marko.W
 * @createDate: 2021-06-16 10:39
 * @version: 1.0
 * @todo:
 */
@Slf4j
@Component
public class TableStoreBatchInsertConsumer{

    public static final String TABLE_STORE_BATCH_INSERT_QUEUE = "qms_table_store_queue";

    @Autowired
    private TableStorePlusUtils tableStorePlusUtils;

    @RabbitHandler
    @SneakyThrows
    @RabbitListener(queues = TABLE_STORE_BATCH_INSERT_QUEUE,ackMode = "AUTO",concurrency = "10-50")
    public void onMessage(Message message){
        try {
            String messageInfo = new String(message.getBody(), "utf-8");
            operateTableStore(messageInfo);
        } catch (Exception e) {
            log.warn("保存码到TableStore异常:{}", e);
            //fixme 这里还要处理失败的请求
        }
    }

    private void operateTableStore(String messageInfo) {
        List<QrCode> qrCodes = JSONUtil.toList(messageInfo,QrCode.class);
        tableStorePlusUtils.putRow(qrCodes);
    }

}
