package com.abi.qms.platform.infrastructure.qr.build.parent;

import java.util.Map;

/**
 * 流水号方式-构建码
 *
 * @author WeiganCai
 * @date 2021-04-19
 */
public abstract class SerialBuildCode extends BuildCode {

    /** paramMap中存放流水号的key */
    public static final String SERIAL_KEY = "serial";

    /** 流水号变量名称的前缀 */
    public static final String SERIAL_START_WITH = "SD";

    /**
     * 生成码
     * @param paramMap 参数map，用于接收生成码要用的参数
     * @return 生成好的码
     */
    @Override
    public String buildCode(Map<String, Object> paramMap) {
        String serialKey = getSerialKey();
        if (paramMap == null || paramMap.get(serialKey) == null) {
            return structCode(0L);
        }
        return structCode(Long.parseLong(paramMap.get(serialKey).toString()));
    }

    /**
     * 对counter加1，并格式化成指定的位数
     * @param counter
     * @return
     */
    private String structCode(long counter) {
        // 获取流水号长度
        int serialLength = getSerialLength();

        // 计算该长度下的最大值
        StringBuffer maxStr = new StringBuffer();
        for (int i = 0; i < serialLength; i++) {
            maxStr.append("9");
        }
        Long max = Long.parseLong(maxStr.toString());

        // 如果超出最大值，从0开始
        if(counter >= max){
            counter = 0L;
        }

        counter++;

        // 在counter之前补零：0-前面补充0，serialLength-长度，d-参数为正数
        return String.format("%0" + serialLength + "d", counter);
    }

    /**
     * 获取流水号长度
     * @return 流水号长度
     */
    protected abstract int getSerialLength();

    /**
     * 获取流水号key
     */
    protected abstract String getSerialKey();

}