package com.abi.qms.platform.infrastructure.rabbitmq.config;


import com.abi.qms.platform.infrastructure.rabbitmq.receiver.DemoMqReceiver;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @author congcong
 */
//@Configuration
@Slf4j
public class DemoMq {

	//新增加MQ修改下面这两个即可
	/**
	 * 业务前缀
	 */
	private static final String MQ_PREFIX = "demo";

	/**
	 * MQ消费对象
	 */
	@Autowired
	private DemoMqReceiver receiver;

	//根据规则拼接出队列和交换器

	public static final String EXCHANGE_NAME = MQ_PREFIX+"_callback_exchange";

	public static final String QUEUE_NAME = MQ_PREFIX+"_callback_queue";

	public static final String BINDING_NAME = MQ_PREFIX+"_callback_binding";

	//往容器放入交换器、队列、绑定

	@Bean(EXCHANGE_NAME)
	public FanoutExchange fanoutExchange() {
		return ExchangeBuilder.fanoutExchange(EXCHANGE_NAME).durable(true).build();
	}

	@Bean(QUEUE_NAME)
	public Queue callbackQueue() {
		return new Queue(QUEUE_NAME, true, false, false);
	}

	@Bean(BINDING_NAME)
	public Binding callBackQueueBinding(@Qualifier(QUEUE_NAME) Queue queue, @Qualifier(EXCHANGE_NAME) FanoutExchange exchange) {
		return BindingBuilder.bind(queue).to(exchange);
	}

	@Bean
	public SimpleMessageListenerContainer listenerContainer(@Qualifier(QUEUE_NAME) Queue queue,
															@Qualifier("firstConnectionFactory") CachingConnectionFactory connectionFactory) {
		SimpleMessageListenerContainer container = getContainer(connectionFactory);
		container.setQueues(queue);
		container.setMessageListener(receiver);
		return container;
	}

	private SimpleMessageListenerContainer getContainer(@Qualifier("firstConnectionFactory") CachingConnectionFactory firstConnectionFactory) {
		SimpleMessageListenerContainer container = new SimpleMessageListenerContainer(firstConnectionFactory);
		container.setConcurrentConsumers(1);
		container.setMaxConcurrentConsumers(2);
		container.setPrefetchCount(0);
		container.setAcknowledgeMode(AcknowledgeMode.MANUAL);
		return container;
	}
}
