package com.abi.qms.platform.infrastructure.rabbitmq.receiver;

import cn.hutool.json.JSONUtil;
import com.abi.qms.platform.dao.entity.UserInfo;
import com.abi.qms.platform.infrastructure.rabbitmq.config.DemoMq;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.listener.api.ChannelAwareMessageListener;
import org.springframework.stereotype.Component;

/**
 * @author 接受MQ消费demo
 */

//@Component
@Slf4j
public class DemoMqReceiver implements ChannelAwareMessageListener {

    @Override
    @RabbitListener(queues = DemoMq.QUEUE_NAME)
    public void onMessage(Message message, Channel channel) throws Exception {
        long deliveryTag = message.getMessageProperties().getDeliveryTag();
        try {
            String messageInfo = new String(message.getBody(), "utf-8");
            doReceiveMessage(messageInfo);
            channel.basicAck(deliveryTag, false);
        } catch (Exception e) {
            log.error("DEMO异常:{}", e);
            channel.basicAck(deliveryTag, false);
        }
    }

    /**
     * 业务代码
     *
     * @param messageInfo
     */
    private void doReceiveMessage(String messageInfo) {
        UserInfo userInfo = JSONUtil.toBean(messageInfo, UserInfo.class);
        log.info("DEMO接到MQ信息拉----》" + messageInfo);
        log.info("转对象----》" + userInfo);
    }

}
