package com.abi.qms.platform.infrastructure.util;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

/**
 * 生成码工具类
 *
 * @author WeiganCai
 * @date 2021-04-19
 */
@Slf4j
public class BuildCodeUtil {

    /** 生成码所在类的包路径 */
    private static final String CLASSPATH = "com.abi.qms.platform.infrastructure.qr.build.children.";

    /**
     * 构造单个的码
     * @param buildCodeClass
     * @param paramMap
     * @return
     */
    public static String buildSingleCode(String buildCodeClass, Map<String, Object> paramMap) {
        String code = "";
        try {
            Class<?> clazz = Class.forName(CLASSPATH + buildCodeClass);
            code = (String) clazz.getMethod("buildCode", Map.class).invoke(clazz.newInstance(), paramMap);
        } catch (Exception e) {
            log.error("buildSingleCode error", e);
        }
        return code;
    }

    /**
     * 构建码
     */
    public static String buildCodes(String buildCodeClasses, String split, Map<String, Object> paramMap) {
        Class<?> clazz;
        String[] buildCodeClassArr = buildCodeClasses.split(",");
        StringBuilder str = new StringBuilder();
        for (int i = 0; i < buildCodeClassArr.length; i++) {
            try {
                clazz = Class.forName(CLASSPATH + buildCodeClassArr[i]);
                str.append(clazz.getMethod("buildCode", Map.class).invoke(clazz.newInstance(), paramMap));
                if (StringUtils.isNotBlank(split)) {
                    str.append(split);
                }
            } catch (Exception e) {
                log.error("BuildCodeUtils->buildCodes->error", e);
            }
        }
        if (StringUtils.isNotBlank(split)) {
            str.deleteCharAt(str.length() - 1);
        }
        return str.toString();
    }

    /**
     * 获得最大不重复数量
     */
    public static Long getMaxCount(String buildCodeClass, Map<String, Object> paramMap) {
        Long maxCount = 1L;
        Class clazz;
        try {
            clazz = Class.forName(CLASSPATH + buildCodeClass);
            maxCount = (Long) clazz.getMethod("getMaxCount", Map.class).invoke(clazz.newInstance(), paramMap);
        } catch (Exception e) {
            log.error("BuildCodeUtils->getMaxCount->error", e);
        }
        return maxCount;
    }

}
