package com.abi.qms.platform.infrastructure.util;


import com.abi.base.foundation.exception.BusinessException;
import com.abi.qms.platform.dao.entity.UserInfo;
import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import lombok.extern.slf4j.Slf4j;

import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;

import static com.abi.qms.platform.infrastructure.constant.RsaKey.RSA_PRIVATE_KEY;
import static com.abi.qms.platform.infrastructure.constant.RsaKey.RSA_PUBLIC_KEY;
import static com.abi.qms.platform.runner.CustomApplicationRunner.globalRsaKeyMap;

/**
 * @className: com.abi.crm.infrastructure.utils-> JwtTokenUtils
 * @description:
 * @author: Marko.W
 * @createDate: 2021-05-17 18:52
 * @version: 1.0
 * @todo:
 */
@Slf4j
public class JwtTokenUtils{


    public static Boolean checkToken(String token){
        try{
            JWTVerifier jwtVerifier = JWT.require(Algorithm.RSA512((RSAPublicKey)globalRsaKeyMap.get(RSA_PUBLIC_KEY),
                    (RSAPrivateKey)globalRsaKeyMap.get(RSA_PRIVATE_KEY))).build();
            jwtVerifier.verify(token);
            return true;
        }catch (Exception e){
            log.info("token校验失败",e);
            throw new BusinessException(401,"token认证失败");
        }
    }

    /**
     * 根据console端user用户生成token信息
     * @param user
     * @return
     */
    public static String getToken(UserInfo user){
        try {
            String token = JWT.create()
                    .withKeyId(UUIDutils.getUUID())
                    .withAudience(user.getId().toString())
                    .withClaim("userId",user.getId())
                    .withClaim("userName",user.getUserName())
                    .withClaim("roleCode",user.getRoleCode())
                    .withClaim("roles","")
                    .sign(Algorithm.RSA512((RSAPublicKey)globalRsaKeyMap.get(RSA_PUBLIC_KEY),
                            (RSAPrivateKey)globalRsaKeyMap.get(RSA_PRIVATE_KEY)));
            return token;
        } catch (Exception e) {
            log.warn("生成token异常",e);
            throw new BusinessException(500,"生成token异常");
        }
    }
}
