package com.abi.qms.platform.infrastructure.util;

import cn.hutool.core.io.FileUtil;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.springframework.core.io.ClassPathResource;
import org.springframework.security.rsa.crypto.KeyStoreKeyFactory;

import javax.crypto.Cipher;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.security.*;
import java.security.interfaces.RSAKey;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.Map;

import static com.abi.qms.platform.infrastructure.constant.RsaKey.RSA_PRIVATE_KEY;
import static com.abi.qms.platform.infrastructure.constant.RsaKey.RSA_PUBLIC_KEY;


/**
 * @className: com.abi.crm.infrastructure.utils-> RSAUtils
 * @description:
 * @author: Marko.W
 * @createDate: 2021-05-18 14:12
 * @version: 1.0
 * @todo:
 */
@Slf4j
public class RSAUtils {

    private  final static String publicKeyPath = "static/publicKey.txt";
    private  final static String privateKeyPath = "static/cxs-jwt.jks";


    private static String getKey(String filename) throws IOException {
        ClassPathResource resourcePublicKey = new ClassPathResource(filename);
        try {
            return FileUtil.readString(resourcePublicKey.getFile(), Charset.defaultCharset());
        } catch (IOException e) {
            log.warn("读取文件失败",e);
            throw e;
        }
    }
    public static Map<String, RSAKey> getKey() throws IOException, GeneralSecurityException {
        Map<String,RSAKey> map = Maps.newHashMap();
        ClassPathResource resource = new ClassPathResource(privateKeyPath);
        KeyStoreKeyFactory keyStoreKeyFactory = new KeyStoreKeyFactory(resource, "1qaz#EDC".toCharArray());
        //密钥对（公钥对应私钥）
        KeyPair keyPair = keyStoreKeyFactory.getKeyPair("cxs-jwt");
        map.put(RSA_PUBLIC_KEY,(RSAPublicKey)keyPair.getPublic());
        map.put(RSA_PRIVATE_KEY,(RSAPrivateKey) keyPair.getPrivate());
        return map;
    }

    public static RSAPrivateKey getPrivateKeyFromString(String key) throws IOException, GeneralSecurityException {
        String privateKeyPEM = key;
        privateKeyPEM = privateKeyPEM.replace("-----BEGIN PRIVATE KEY-----\n", "");
        privateKeyPEM = privateKeyPEM.replace("-----END PRIVATE KEY-----", "");
        byte[] encoded = Base64.decodeBase64(privateKeyPEM);
        KeyFactory kf = KeyFactory.getInstance("RSA");
        PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(encoded);
        return (RSAPrivateKey) kf.generatePrivate(keySpec);
    }


    public static RSAPublicKey getPublicKey() throws IOException, GeneralSecurityException {
        return getPublicKeyFromString(getKey(publicKeyPath));
    }

    public static RSAPublicKey getPublicKeyFromString(String key) throws IOException, GeneralSecurityException {
        String publicKeyPEM = key;
        publicKeyPEM = publicKeyPEM.replace("-----BEGIN PUBLIC KEY-----\n", "");
        publicKeyPEM = publicKeyPEM.replace("-----END PUBLIC KEY-----", "");
        byte[] encoded = Base64.decodeBase64(publicKeyPEM);
        KeyFactory kf = KeyFactory.getInstance("RSA");
        return (RSAPublicKey) kf.generatePublic(new X509EncodedKeySpec(encoded));
    }



    public static String sign(PrivateKey privateKey, String message) throws NoSuchAlgorithmException, InvalidKeyException, SignatureException, UnsupportedEncodingException {
        Signature sign = Signature.getInstance("SHA1withRSA");
        sign.initSign(privateKey);
        sign.update(message.getBytes("UTF-8"));
        return new String(Base64.encodeBase64(sign.sign()), "UTF-8");
    }


    public static boolean verify(PublicKey publicKey, String message, String signature) throws SignatureException, NoSuchAlgorithmException, UnsupportedEncodingException, InvalidKeyException {
        Signature sign = Signature.getInstance("SHA1withRSA");
        sign.initVerify(publicKey);
        try {
            sign.update(message.getBytes("UTF-8"));
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return sign.verify(Base64.decodeBase64(signature.getBytes("UTF-8")));
    }

    public static String encrypt(String rawText, PublicKey publicKey) throws IOException, GeneralSecurityException {
        Cipher cipher = Cipher.getInstance("RSA");
        cipher.init(Cipher.ENCRYPT_MODE, publicKey);
        return Base64.encodeBase64String(cipher.doFinal(rawText.getBytes("UTF-8")));
    }

    public static String decrypt(String cipherText, PrivateKey privateKey) throws IOException, GeneralSecurityException {
        Cipher cipher = Cipher.getInstance("RSA");
        cipher.init(Cipher.DECRYPT_MODE, privateKey);
        return new String(cipher.doFinal(Base64.decodeBase64(cipherText)), "UTF-8");
    }

}
