package com.abi.qms.platform.service;

import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.ActivateDetailRes;
import com.abi.qms.platform.dto.res.ActivationRecordRes;
import com.abi.qms.platform.dto.res.ListQrBoxCodeMappingRes;
import com.abi.qms.platform.dto.res.PrintingDetailRes;
import com.abi.qms.platform.dto.res.QrBoxCodeUploadRes;
import com.abi.qms.platform.dto.res.QueryWxQrBoxMappingDetailsWxRes;
import com.abi.task.common.excel.preperties.QrBoxExportProperty;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.Map;

/**
 * @author: fangxinjian
 * @date: 2021/05/27 11:18
 * @description:
 */
public interface QrBoxMappingService {

    /**
     * 生成条形码
     *
     * @param generateBarCodeReq
     */
    String generateBarCode(GenerateBarCodeReq generateBarCodeReq);

    /**
     * 分页查询箱码合一
     *
     * @param listQrBoxCodeMappingReq
     * @return
     */
    ListQrBoxCodeMappingRes listQrBoxCodeMapping(ListQrBoxCodeMappingReq listQrBoxCodeMappingReq);

    /**
     * 箱码合一文件上传
     *
     * @param map
     * @return
     */
    QrBoxCodeUploadRes uploadCodeFiles(Map<String, MultipartFile> map, Long packageId);

    /**
     * 根据ID激活码包
     *
     * @param activeBoxCodeReq
     */
    void activeBoxCode(ActiveBoxCodeReq activeBoxCodeReq);

    /**
     * 根据码包ID查询箱码合一打印详情
     *
     * @param printingDetailReq
     * @return
     */
    PrintingDetailRes getPrintingDetail(PrintingDetailReq printingDetailReq);

    /**
     * 根据ID作废码包
     *
     * @param activeBoxCodeReq
     */
    void invalidBoxCode(ActiveBoxCodeReq activeBoxCodeReq);


    /**
     * 微信 - 箱码/条码详情
     *
     * @param queryWxQrBoxMappingDetailsWxReq
     * @return com.abi.qms.platform.dto.res.QueryWxQrBoxMappingDetailsWxRes
     * @author lu
     * @date 2021/6/9 11:24
     */
    QueryWxQrBoxMappingDetailsWxRes queryWxQrBoxMappingDetails(QueryWxQrBoxMappingDetailsWxReq queryWxQrBoxMappingDetailsWxReq);

    /**
     * 补充激活
     *
     * @param replenishActivationReq
     */
    void replenishActivation(ReplenishActivationReq replenishActivationReq);

    /**
     * 激活记录
     *
     * @param activationRecordReq
     * @return
     */
    ActivationRecordRes getActivationRecord(ActivationRecordReq activationRecordReq);

    /**
     * 立即激活
     *
     * @param activateNowReq
     */
    void activateNow(ActivateNowReq activateNowReq);

    /**
     * 激活详情
     *
     * @param activateDetailReq
     * @return
     */
    ActivateDetailRes getActivateDetail(ActivateDetailReq activateDetailReq);

    /**
     * 导出数据
     * @return
     */
    List<QrBoxExportProperty> downloadExcelData(ListQrBoxCodeMappingExportReq req);
}