package com.abi.qms.platform.service;


import com.abi.qms.platform.dao.entity.QrBoxMapping;
import com.abi.qms.platform.dao.entity.QrPackage;
import com.abi.qms.platform.dao.tablestore.entity.QrCode;
import com.abi.qms.platform.dto.req.DeleteQrPackageReq;
import com.abi.qms.platform.dto.req.DownloadQrPackageReq;
import com.abi.qms.platform.dto.req.GetPackageCodeReq;
import com.abi.qms.platform.dto.req.GetQrPackageDetailReq;
import com.abi.qms.platform.dto.req.GetQrSingleCheckDetailCodeReq;
import com.abi.qms.platform.dto.req.InvalidQrPackageReq;
import com.abi.qms.platform.dto.req.InvalidQrSingleReq;
import com.abi.qms.platform.dto.req.ListQrPackageReq;
import com.abi.qms.platform.dto.req.ListQrPackageWxReq;
import com.abi.qms.platform.dto.req.PassQrPackageReq;
import com.abi.qms.platform.dto.req.RefuseQrPackageReq;
import com.abi.qms.platform.dto.req.SaveQrPackageReq;
import com.abi.qms.platform.dto.req.SendMobileMsgReq;
import com.abi.qms.platform.dto.req.ValidSAPOrderNoAndMaterialReq;
import com.abi.qms.platform.dto.req.WxQrPackageDetailReq;
import com.abi.qms.platform.dto.res.GetPackageCodeRes;
import com.abi.qms.platform.dto.res.GetQrPackageDetailRes;
import com.abi.qms.platform.dto.res.GetQrSingleCheckDetailRes;
import com.abi.qms.platform.dto.res.ListQrPackageRes;
import com.abi.qms.platform.dto.res.ListQrPackageWxRes;
import com.abi.qms.platform.dto.res.ValidSAPOrderNoAndMaterialRes;
import com.abi.qms.platform.dto.res.WxQrPackageDetailRes;

import javax.servlet.http.HttpServletResponse;

/**
 * 码包 Service接口
 *
 * @author WeiganCai
 * @date 2021-04-28
 */
public interface QrPackageService {

    /**
     * 保存码包
     */
    void saveQrPackage(SaveQrPackageReq saveQrPackageReq);

    /**
     * 码包分页查询
     */
    ListQrPackageRes listQrPackage(ListQrPackageReq listQrPackageReq);

    /**
     * 码包查询详情
     */
    GetQrPackageDetailRes getQrPackageDetail(GetQrPackageDetailReq getQrPackageDetailReq);

    /**
     * 删除码包
     */
    void deleteQrPackage(DeleteQrPackageReq deleteQrPackageReq);

    /**
     * 审核通过码包
     */
    void passQrPackage(PassQrPackageReq passQrPackageReq);

    /**
     * 审核拒绝码包
     */
    void refuseQrPackage(RefuseQrPackageReq refuseQrPackageReq);

    /**
     * 下载码包
     */
    void downloadQrPackage(DownloadQrPackageReq downloadQrPackageReq, HttpServletResponse response);

    /**
     * 码生成失败回滚
     */
    void failedRollback(QrPackage qrPackage);

    /**
     * 码包生成完成后发送邮件给码包厂负责人邮箱
     *
     * @param targetEmail
     * @param qrPackage
     * @param filename
     * @param password
     */
    void sendEmailForQrPackageCreated(String targetEmail, QrPackage qrPackage, String filename, String password);

    /**
     * 校验是否存在SAP订单号和物料重复数据
     *
     * @param req
     * @return
     */
    ValidSAPOrderNoAndMaterialRes validSAPOrderNoAndMaterial(ValidSAPOrderNoAndMaterialReq req);

    /**
     * 发送短信验证码
     *
     * @param req
     */
    void sendMobileMsg(SendMobileMsgReq req);

    /**
     * 单码查询详情
     */
    GetQrSingleCheckDetailRes getQrSingleCheckDetail(GetQrSingleCheckDetailCodeReq getQrSingleCheckDetailCodeReq);


    /**
     * 作废码包
     *
     * @param invalidQrPackageReq
     */
    void invalidQrPackage(InvalidQrPackageReq invalidQrPackageReq);


    /**
     * 根据主键code修改作废状态
     *
     * @param invalidQrSingleReq
     */
    void invalidQrSingle(InvalidQrSingleReq invalidQrSingleReq);

    /**
     * 根据物料ID获取码包code
     *
     * @param req
     * @return
     */
    GetPackageCodeRes getPackageCodeType(GetPackageCodeReq req);

    /**
     * 微信小程序查询码包列表
     *
     * @param listQrPackageWxReq
     * @return
     */
    ListQrPackageWxRes listWxQrPackage(ListQrPackageWxReq listQrPackageWxReq);

    /**
     * 微信小程序查询码包详情
     *
     * @param wxQrPackageDetailReq
     * @return
     */
    WxQrPackageDetailRes getWxQrPackageDetail(WxQrPackageDetailReq wxQrPackageDetailReq);

    /**
     * 查看码是否作废
     * @param qrCode
     * @return
     */
    boolean getPackeageInvalid (String qrCode);


    /**
     * 根据码数据获取激活信息
     * @param qrCode 码对象
     * @return
     */
    QrBoxMapping getPackeageActivation (QrCode qrCode);

}
