package com.abi.qms.platform.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.abi.qms.platform.dao.entity.BaseSapBrand;
import com.abi.qms.platform.dao.mapper.BaseSapBrandMapper;
import com.abi.qms.platform.dto.req.BatchImportBrandReq;
import com.abi.qms.platform.service.BaseSapBrandService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author: fangxinjian
 * @date: 2021/06/02 10:26
 * @description:
 */
@Service
@Slf4j
public class BaseSapBrandServiceImpl implements BaseSapBrandService {

    @Autowired
    private BaseSapBrandMapper baseSapBrandMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchImportBrand(BatchImportBrandReq req) {
        //sap品牌数据是全量数据，先删除品牌数据
        baseSapBrandMapper.deleteTable();
        List<BatchImportBrandReq.SapBrandBean> brandBeanList = req.getSapBrandBeanList();

        for (BatchImportBrandReq.SapBrandBean brand : brandBeanList) {

            try {
                //申明对象
                BaseSapBrand sapBrand = new BaseSapBrand();
                sapBrand.setBrandCode(brand.getBrandCode());
                sapBrand.setBrandLevel(brand.getBrandLevel());
                sapBrand.setBrandName(brand.getBrandName());
                sapBrand.setParentBrandCode(brand.getParentBrandCode());
                sapBrand.setParentBrandName(brand.getParentBrandName());

                saveBaseSapBrand(sapBrand);
            } catch (Exception e) {
                log.error("导入Sap品牌数据失败", e);
            }
        }
    }

    @Override
    public void batchUpdateBrand(BatchImportBrandReq req) {

        List<BatchImportBrandReq.SapBrandBean> brandBeanList = req.getSapBrandBeanList();

        for (BatchImportBrandReq.SapBrandBean brand : brandBeanList) {

            try {
                //申明对象
                BaseSapBrand sapBrand = new BaseSapBrand();
                sapBrand.setBrandCode(brand.getBrandCode());
                sapBrand.setBrandLevel(brand.getBrandLevel());
                sapBrand.setBrandName(brand.getBrandName());
                sapBrand.setParentBrandCode(brand.getParentBrandCode());
                sapBrand.setParentBrandName(brand.getParentBrandName());

                updateBaseSapBrand(sapBrand);
            } catch (Exception e) {
                log.error("修改Sap品牌数据失败", e);
            }
        }

    }

    private void updateBaseSapBrand(BaseSapBrand bsb) {

        BaseSapBrand sapBrand = getBaseSapBrand(bsb.getBrandCode());

        if (ObjectUtil.isNull(bsb)) {
            log.info("品牌code:{}不存在", bsb.getBrandCode());
            return;
        }

        bsb.setId(sapBrand.getId());
        baseSapBrandMapper.updateById(bsb);
    }

    public void saveBaseSapBrand(BaseSapBrand bsb) {

        BaseSapBrand sapBrand = getBaseSapBrand(bsb.getBrandCode());

        if (ObjectUtil.isNotNull(sapBrand)) {
            log.info("品牌code:{}已存在", bsb.getBrandCode());
            return;
        }

        baseSapBrandMapper.insert(bsb);

    }

    private BaseSapBrand getBaseSapBrand(String brandCode) {

        QueryWrapper<BaseSapBrand> qw = new QueryWrapper<>();
        qw.eq("brand_code", brandCode);
        qw.eq("is_delete", 0);

        //根据品牌code查询Sap品牌信息
        BaseSapBrand baseSapBrand = baseSapBrandMapper.selectOne(qw);

        return baseSapBrand;

    }
}
