package com.abi.qms.platform.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.abi.qms.platform.dao.entity.BaseSapCoverFactory;
import com.abi.qms.platform.dao.mapper.BaseSapCoverFactoryMapper;
import com.abi.qms.platform.dto.req.BatchImportCoverFactoryReq;
import com.abi.qms.platform.service.BaseSapCoverFactoryService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author: fangxinjian
 * @date: 2021/06/04 15:23
 * @description:
 */
@Service
@Slf4j
public class BaseSapCoverFactoryServiceImpl implements BaseSapCoverFactoryService {

    @Autowired
    private BaseSapCoverFactoryMapper baseSapCoverFactoryMapper;

    @Override
    public void batchImportCoverFactory(BatchImportCoverFactoryReq req) {

        List<BatchImportCoverFactoryReq.BatchImportCoverFactory> sapCoverFactoryList = req.getBatchImportCoverFactoryList();

        for (BatchImportCoverFactoryReq.BatchImportCoverFactory coverFactory : sapCoverFactoryList) {

            try {
                BaseSapCoverFactory baseSapCoverFactory = new BaseSapCoverFactory();
                baseSapCoverFactory.setSapCoverFactoryId(coverFactory.getSapCoverFactoryId());
                baseSapCoverFactory.setSapCoverFactoryName(coverFactory.getSapCoverFactoryName());
                baseSapCoverFactory.setKeymanName(coverFactory.getKeymanName());
                baseSapCoverFactory.setKeymanMobile(coverFactory.getKeymanMobile());

                saveBaseSapCoverFactory(baseSapCoverFactory);
            } catch (Exception e) {
                log.error("导入Sap供应商数据失败", e);
            }
        }

    }

    @Override
    public void batchUpdateCoverFactory(BatchImportCoverFactoryReq req) {

        List<BatchImportCoverFactoryReq.BatchImportCoverFactory> sapCoverFactoryList = req.getBatchImportCoverFactoryList();

        for (BatchImportCoverFactoryReq.BatchImportCoverFactory coverFactory : sapCoverFactoryList) {

            try {
                BaseSapCoverFactory baseSapCoverFactory = new BaseSapCoverFactory();
                baseSapCoverFactory.setSapCoverFactoryId(coverFactory.getSapCoverFactoryId());
                baseSapCoverFactory.setSapCoverFactoryName(coverFactory.getSapCoverFactoryName());
                baseSapCoverFactory.setKeymanName(coverFactory.getKeymanName());
                baseSapCoverFactory.setKeymanMobile(coverFactory.getKeymanMobile());

                updateBaseSapCoverFactory(baseSapCoverFactory);
            } catch (Exception e) {
                log.error("修改Sap供应商数据失败", e);
            }
        }

    }

    /**
     * 更新Sap供应商信息
     *
     * @param sapCoverFactory
     */
    private void updateBaseSapCoverFactory(BaseSapCoverFactory sapCoverFactory) {

        //根据sap供应商ID查询供应商信息
        BaseSapCoverFactory baseSapCoverFactory = getBaseSapCoverFactoryInfo(sapCoverFactory.getSapCoverFactoryId());

        if (ObjectUtil.isNull(baseSapCoverFactory)) {
            log.info("Sap供应商ID:{}不存在", baseSapCoverFactory.getSapCoverFactoryId());
            return;
        }

        sapCoverFactory.setId(baseSapCoverFactory.getId());
        sapCoverFactory.setUpdateTime(LocalDateTime.now());
        baseSapCoverFactoryMapper.updateById(sapCoverFactory);

    }


    /**
     * 保存Sap供应商信息
     *
     * @param sapCoverFactory
     */
    private void saveBaseSapCoverFactory(BaseSapCoverFactory sapCoverFactory) {

        //根据sap供应商ID查询供应商信息
        BaseSapCoverFactory baseSapCoverFactory = getBaseSapCoverFactoryInfo(sapCoverFactory.getSapCoverFactoryId());

        if (ObjectUtil.isNotNull(baseSapCoverFactory)) {
            log.info("Sap供应商ID:{}已存在", baseSapCoverFactory.getSapCoverFactoryId());
            return;
        }

        baseSapCoverFactoryMapper.insert(sapCoverFactory);
    }


    /**
     * 根据sap供应商ID查询sap供应商信息
     *
     * @param sapCoverFactoryId sap供应商ID
     * @return
     */
    private BaseSapCoverFactory getBaseSapCoverFactoryInfo(String sapCoverFactoryId) {

        QueryWrapper<BaseSapCoverFactory> qw = new QueryWrapper<>();
        qw.eq("sap_cover_factory_id", sapCoverFactoryId);
        qw.eq("is_delete", 0);

        //根据sap供应商id查询供应商信息
        BaseSapCoverFactory baseSapCoverFactory = baseSapCoverFactoryMapper.selectOne(qw);

        return baseSapCoverFactory;

    }


}
