package com.abi.qms.platform.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.abi.qms.platform.dao.entity.BaseSapMaterial;
import com.abi.qms.platform.dao.mapper.BaseSapMaterialMapper;
import com.abi.qms.platform.dto.req.BatchImportMaterialReq;
import com.abi.qms.platform.service.BaseSapMaterialService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author: fangxinjian
 * @date: 2021/06/04 11:40
 * @description:
 */
@Service
@Slf4j
public class BaseSapMaterialServiceImpl implements BaseSapMaterialService {

    @Autowired
    private BaseSapMaterialMapper baseSapMaterialMapper;

    @Override
    public void batchImportMaterial(BatchImportMaterialReq req) {
        List<BatchImportMaterialReq.BatchImportMaterial> sapMaterialList = req.getBatchImportMaterialList();

        for (BatchImportMaterialReq.BatchImportMaterial material : sapMaterialList) {

            try {
                BaseSapMaterial baseSapMaterial = new BaseSapMaterial();
                baseSapMaterial.setSapMaterialId(material.getSapMaterialId());
                baseSapMaterial.setSapMaterialName(material.getSapMaterialName());

                saveBaseSapMaterial(baseSapMaterial);
            } catch (Exception e) {
                log.error("导入Sap物料数据失败", e);
            }
        }

    }

    @Override
    public void batchUpdateMaterial(BatchImportMaterialReq req) {
        List<BatchImportMaterialReq.BatchImportMaterial> sapMaterialList = req.getBatchImportMaterialList();

        for (BatchImportMaterialReq.BatchImportMaterial material : sapMaterialList) {

            try {
                BaseSapMaterial baseSapMaterial = new BaseSapMaterial();
                baseSapMaterial.setSapMaterialId(material.getSapMaterialId());
                baseSapMaterial.setSapMaterialName(material.getSapMaterialName());

                updateBaseSapMaterial(baseSapMaterial);
            } catch (Exception e) {
                log.error("修改Sap物料数据失败", e);
            }
        }

    }

    /**
     * 更新Sap物料信息
     *
     * @param sapMaterial
     */
    private void updateBaseSapMaterial(BaseSapMaterial sapMaterial) {

        //根据sap物料ID查询物料信息
        BaseSapMaterial baseSapMaterial = getBaseSapMaterialInfo(sapMaterial.getSapMaterialId());

        if (ObjectUtil.isNull(baseSapMaterial)) {
            log.info("Sap物料ID:{}不存在", baseSapMaterial.getSapMaterialId());
            return;
        }

        sapMaterial.setId(baseSapMaterial.getId());
        sapMaterial.setUpdateTime(LocalDateTime.now());
        baseSapMaterialMapper.updateById(sapMaterial);

    }


    /**
     * 保存Sap物料信息
     *
     * @param sapMaterial
     */
    private void saveBaseSapMaterial(BaseSapMaterial sapMaterial) {

        //根据sap物料ID查询物料信息
        BaseSapMaterial baseSapMaterial = getBaseSapMaterialInfo(sapMaterial.getSapMaterialId());

        if (ObjectUtil.isNotNull(baseSapMaterial)) {
            log.info("Sap物料ID:{}已存在", baseSapMaterial.getSapMaterialId());
            return;
        }

        baseSapMaterialMapper.insert(sapMaterial);
    }


    private BaseSapMaterial getBaseSapMaterialInfo(String sapMaterialId) {

        QueryWrapper<BaseSapMaterial> qw = new QueryWrapper<>();
        qw.eq("sap_material_id", sapMaterialId);
        qw.eq("is_delete", 0);

        //根据sap物料id查询物料信息
        BaseSapMaterial baseSapMaterial = baseSapMaterialMapper.selectOne(qw);

        return baseSapMaterial;

    }


}
