package com.abi.qms.platform.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.abi.qms.platform.dao.entity.BaseSapOrganization;
import com.abi.qms.platform.dao.mapper.BaseSapOrganizationMapper;
import com.abi.qms.platform.dto.req.BatchImportOrganizationReq;
import com.abi.qms.platform.service.BaseSapOrganizationService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author: fangxinjian
 * @date: 2021/06/01 17:20
 * @description:
 */
@Service
@Slf4j
public class BaseSapOrganizationServiceImpl implements BaseSapOrganizationService {

    @Autowired
    private BaseSapOrganizationMapper baseSapOrganizationMapper;


    @Override
    public void batchImportOrganization(BatchImportOrganizationReq req) {
        List<BatchImportOrganizationReq.SapOrganization> organizationList = req.getOrganizationList();

        for (BatchImportOrganizationReq.SapOrganization sap : organizationList) {

            try {
                BaseSapOrganization organization = new BaseSapOrganization();
                organization.setCustomerCode(sap.getCustomerCode());
                organization.setCustomerName(sap.getCustomerName());
                organization.setOrganizationLevel(sap.getOrganizationLevel());
                organization.setParentCode(sap.getParentCode());
                organization.setProvince(sap.getProvince());

                saveBaseSapOrganization(organization);
            } catch (Exception e) {
                log.error("导入组织架构数据失败", e);
            }
        }

    }

    @Override
    public void batchUpdateOrganization(BatchImportOrganizationReq req) {
        List<BatchImportOrganizationReq.SapOrganization> organizationList = req.getOrganizationList();

        for (BatchImportOrganizationReq.SapOrganization sap : organizationList) {

            try {
                BaseSapOrganization organization = new BaseSapOrganization();
                organization.setCustomerCode(sap.getCustomerCode());
                organization.setCustomerName(sap.getCustomerName());
                organization.setOrganizationLevel(sap.getOrganizationLevel());
                organization.setParentCode(sap.getParentCode());
                organization.setProvince(sap.getProvince());

                updateBeaseSapOrganization(organization);
            } catch (Exception e) {
                log.error("修改组织架构数据失败", e);
            }

        }

    }

    private void updateBeaseSapOrganization(BaseSapOrganization bso) {

        BaseSapOrganization sapOrganization = getBaseSapOrganization(bso.getCustomerCode());

        if (ObjectUtil.isNull(sapOrganization)) {
            log.info("组织架构代码:{}不存在", bso.getCustomerCode());
            return;
        }

        bso.setId(sapOrganization.getId());
        bso.setUpdateTime(LocalDateTime.now());
        baseSapOrganizationMapper.updateById(bso);
    }

    public void saveBaseSapOrganization(BaseSapOrganization bso) {

        BaseSapOrganization sapOrganization = getBaseSapOrganization(bso.getCustomerCode());

        if (ObjectUtil.isNotNull(sapOrganization)) {
            log.info("组织架构代码:{}已存在", bso.getCustomerCode());
            return;
        }

        baseSapOrganizationMapper.insert(bso);

    }


    private BaseSapOrganization getBaseSapOrganization(String customerCode) {

        QueryWrapper<BaseSapOrganization> qw = new QueryWrapper<>();
        qw.eq("customer_code", customerCode);
        qw.eq("is_delete", 0);

        //根据组织架构代码查询组织架构信息
        BaseSapOrganization baseSapOrganization = baseSapOrganizationMapper.selectOne(qw);

        return baseSapOrganization;

    }
}
