package com.abi.qms.platform.service.impl;

import com.abi.qms.platform.dao.entity.BaseBrand;
import com.abi.qms.platform.dao.mapper.BaseBrandMapper;
import com.abi.qms.platform.dto.req.ListBrandForSelectReq;
import com.abi.qms.platform.dto.res.ListBrandForSelectRes;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.service.BrandService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 品牌管理
 * </p>
 *
 * @author Andy.Tan
 */
@Service
public class BrandServiceImpl implements BrandService {

    @Autowired
    private BaseBrandMapper baseBrandMapper;

    /**
     * 查询品牌信息
     * @param brandCode
     * @return
     */
    @Override
    public BaseBrand getBrand(String brandCode){
        QueryWrapper<BaseBrand> brandQw = new QueryWrapper<>();
        brandQw.eq("brand_code",brandCode);
        brandQw.eq("is_delete",0);
        BaseBrand baseBrand = baseBrandMapper.selectOne(brandQw);
        AssertUtil.isNull(baseBrand,"品牌不存在");

        return baseBrand;
    }

    /**
     * 下拉选择品牌列表
     **/
    @Override
    public ListBrandForSelectRes listBrandForSelect(ListBrandForSelectReq req) {
        //1-查询所有品牌
        QueryWrapper<BaseBrand> baseBrandQw = new QueryWrapper<>();
        baseBrandQw.eq("is_delete",0);
        List<BaseBrand> baseBrandList = baseBrandMapper.selectList(baseBrandQw);

        //出参列表
        List<ListBrandForSelectRes.BrandBean> brandBeanList = new ArrayList<>();

        //2-构造树形结构
        //定义map用来存放父类bean
        Map<String,ListBrandForSelectRes.BrandBean> brandBeanMap = new HashMap<>();
        //分一级和二级两组
        List<BaseBrand> l1BrandList = baseBrandList.stream().filter(brand -> brand.getBrandLevel() == 1).collect(Collectors.toList());
        List<BaseBrand> l2BrandList = baseBrandList.stream().filter(brand -> brand.getBrandLevel() == 2).collect(Collectors.toList());
        //循环放入一级
        for(BaseBrand brand:l1BrandList){
            ListBrandForSelectRes.BrandBean brandBean = new ListBrandForSelectRes.BrandBean();
            brandBean.setBrandCode(brand.getBrandCode());
            brandBean.setBrandName(brand.getBrandName());
            brandBean.setSubBrandBeanList(new ArrayList<>());
            //放入出参列表
            brandBeanList.add(brandBean);
            //放入map
            brandBeanMap.put(brandBean.getBrandCode(),brandBean);
        }
        //循环放入二级
        for(BaseBrand brand:l2BrandList){
            ListBrandForSelectRes.BrandBean parentBrandBean = brandBeanMap.get(brand.getParentBrandCode());
            if(parentBrandBean==null){
                continue;
            }
            //构建bean
            ListBrandForSelectRes.BrandBean brandBean = new ListBrandForSelectRes.BrandBean();
            brandBean.setBrandCode(brand.getBrandCode());
            brandBean.setBrandName(brand.getBrandName());
            //加入子品牌
            parentBrandBean.getSubBrandBeanList().add(brandBean);
        }

        //构造出参
        ListBrandForSelectRes res = new ListBrandForSelectRes();
        res.setBrandBeanList(brandBeanList);

        return res;
    }

}
