package com.abi.qms.platform.service.impl;

import com.abi.qms.platform.dao.entity.BaseSapBeerFactory;
import com.abi.qms.platform.dao.entity.BaseSapCoverFactory;
import com.abi.qms.platform.dao.entity.BaseSapMaterial;
import com.abi.qms.platform.dao.entity.QrPackageSapOrder;
import com.abi.qms.platform.dao.entity.QrPackageSapOrderItem;
import com.abi.qms.platform.dao.mapper.BaseSapBeerFactoryMapper;
import com.abi.qms.platform.dao.mapper.BaseSapCoverFactoryMapper;
import com.abi.qms.platform.dao.mapper.BaseSapMaterialMapper;
import com.abi.qms.platform.dao.mapper.QrPackageSapOrderItemMapper;
import com.abi.qms.platform.dao.mapper.QrPackageSapOrderMapper;
import com.abi.qms.platform.dto.req.ListMaterialBySapOrderReq;
import com.abi.qms.platform.dto.req.ListSapBeerFactoryReq;
import com.abi.qms.platform.dto.req.ListSapCodePackageOrderReq;
import com.abi.qms.platform.dto.req.ListSapCoverFactoryReq;
import com.abi.qms.platform.dto.req.ListSapMaterialReq;
import com.abi.qms.platform.dto.res.ListMaterialBySapOrderRes;
import com.abi.qms.platform.dto.res.ListSapBeerFactoryRes;
import com.abi.qms.platform.dto.res.ListSapCodePackageOrderRes;
import com.abi.qms.platform.dto.res.ListSapCoverFactoryRes;
import com.abi.qms.platform.dto.res.ListSapMaterialRes;
import com.abi.qms.platform.service.SapBaseQueryService;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * sap基础数据查询
 * </p>
 *
 * @author Andy.Tan
 */
@Service
public class SapBaseQueryServiceImpl implements SapBaseQueryService {

    @Autowired
    private BaseSapMaterialMapper baseSapMaterialMapper;

    @Autowired
    private QrPackageSapOrderMapper qrPackageSapOrderMapper;

    @Autowired
    private QrPackageSapOrderItemMapper qrPackageSapOrderItemMapper;

    @Autowired
    private BaseSapCoverFactoryMapper baseSapCoverFactoryMapper;

    @Autowired
    private BaseSapBeerFactoryMapper baseSapBeerFactoryMapper;

    /**
     * 查询SAP物料列表
     **/
    @Override
    public ListSapMaterialRes listSapMaterial(ListSapMaterialReq req) {
        //查询
        QueryWrapper<BaseSapMaterial> sapMaterialQw = new QueryWrapper<>();
        sapMaterialQw.like(StringUtils.isNotBlank(req.getSapMaterialId()), "sap_material_id", "%" + req.getSapMaterialId() + "%");
        sapMaterialQw.eq("is_delete", 0);
        List<BaseSapMaterial> sapMaterialList = baseSapMaterialMapper.selectList(sapMaterialQw);

        //构造出参
        ListSapMaterialRes res = new ListSapMaterialRes();
        List<ListSapMaterialRes.SapMaterialBean> sapMaterialBeanList = PojoConverterUtils.copyList(sapMaterialList, ListSapMaterialRes.SapMaterialBean.class);
        res.setSapMaterialBeanList(sapMaterialBeanList);

        return res;
    }

    /**
     * 查询SAP包材场列表
     **/
    @Override
    public ListSapCoverFactoryRes listSapCoverFactory(ListSapCoverFactoryReq req) {
        //查询
        QueryWrapper<BaseSapCoverFactory> sapCoverFactoryQW = new QueryWrapper<>();
        sapCoverFactoryQW.like(StringUtils.isNotBlank(req.getSapCoverFactoryId()), "sap_cover_factory_id", "%" + req.getSapCoverFactoryId() + "%");
        sapCoverFactoryQW.eq("is_delete", 0);
        List<BaseSapCoverFactory> sapCoverFactoryList = baseSapCoverFactoryMapper.selectList(sapCoverFactoryQW);

        //构造出参
        ListSapCoverFactoryRes res = new ListSapCoverFactoryRes();
        List<ListSapCoverFactoryRes.SapCoverFactoryBean> sapCoverFactoryBeanList = PojoConverterUtils.copyList(sapCoverFactoryList, ListSapCoverFactoryRes.SapCoverFactoryBean.class);
        res.setSapCoverFactoryBeanList(sapCoverFactoryBeanList);

        return res;
    }

    /**
     * 查询SAP啤酒厂列表
     **/
    @Override
    public ListSapBeerFactoryRes listSapBeerFactory(ListSapBeerFactoryReq req) {
        //查询
        QueryWrapper<BaseSapBeerFactory> sapBeerFactoryQW = new QueryWrapper<>();
        sapBeerFactoryQW.like(StringUtils.isNotBlank(req.getSapBeerFactoryId()), "sap_beer_factory_id", "%" + req.getSapBeerFactoryId() + "%");
        sapBeerFactoryQW.eq("is_delete", 0);
        List<BaseSapBeerFactory> sapBeerFactoryList = baseSapBeerFactoryMapper.selectList(sapBeerFactoryQW);

        //构造出参
        ListSapBeerFactoryRes res = new ListSapBeerFactoryRes();
        List<ListSapBeerFactoryRes.SapBeerFactoryBean> sapBeerFactoryBeanList = PojoConverterUtils.copyList(sapBeerFactoryList, ListSapBeerFactoryRes.SapBeerFactoryBean.class);
        res.setSapBeerFactoryBeanList(sapBeerFactoryBeanList);

        return res;
    }

    /**
     * 查询SAP订单列表
     **/
    @Override
    public ListSapCodePackageOrderRes listSapCodePackageOrder(ListSapCodePackageOrderReq req) {
        //查询
        QueryWrapper<QrPackageSapOrder> orderQw = new QueryWrapper<>();
        orderQw.like(StringUtils.isNotBlank(req.getSapOrderNo()), "sap_order_no", "%" + req.getSapOrderNo() + "%");
        orderQw.eq("is_delete", 0);
        List<QrPackageSapOrder> orderList = qrPackageSapOrderMapper.selectList(orderQw);

        //构造出参
        ListSapCodePackageOrderRes res = new ListSapCodePackageOrderRes();
        List<ListSapCodePackageOrderRes.SapOrderBean> sapOrderBeanList = PojoConverterUtils.copyList(orderList, ListSapCodePackageOrderRes.SapOrderBean.class);
        res.setSapOrderBeanList(sapOrderBeanList);

        return res;
    }

    /**
     * 查询SAP订单关联的QMS物料列表
     **/
    @Override
    public ListMaterialBySapOrderRes listMaterialBySapOrder(ListMaterialBySapOrderReq req) {
        //查询
        QueryWrapper<QrPackageSapOrderItem> orderItemQw = new QueryWrapper<>();
        orderItemQw.eq("sap_order_no", req.getSapOrderNo());
        orderItemQw.eq("is_delete", 0);
        List<QrPackageSapOrderItem> sapOrderItemList = qrPackageSapOrderItemMapper.selectList(orderItemQw);

        //构造出参
        ListMaterialBySapOrderRes res = new ListMaterialBySapOrderRes();
        List<ListMaterialBySapOrderRes.OrderItemBean> orderItemBeanList = PojoConverterUtils.copyList(sapOrderItemList, ListMaterialBySapOrderRes.OrderItemBean.class);
        res.setOrderItemBeanList(orderItemBeanList);

        return res;
    }


}
