package com.abi.qms.platform.service.impl;

import com.abi.qms.platform.dao.entity.UserMappingRoleResource;
import com.abi.qms.platform.dao.entity.UserResource;
import com.abi.qms.platform.dao.entity.UserRole;
import com.abi.qms.platform.dao.mapper.UserMappingRoleResourceMapper;
import com.abi.qms.platform.dao.mapper.UserResourceMapper;
import com.abi.qms.platform.dao.mapper.UserRoleMapper;
import com.abi.qms.platform.service.RoleManagerService;
import com.abi.qms.platform.service.UserResourceService;
import com.abi.task.common.utils.ICollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 页面资源管理
 * @author Andy.Tan
 */
@Service
public class UserResourceServiceImpl extends ServiceImpl<UserMappingRoleResourceMapper, UserMappingRoleResource> implements UserResourceService {

    @Autowired
    private UserResourceMapper userResourceMapper;

    @Autowired
    private UserMappingRoleResourceMapper userMappingRoleResourceMapper;

    @Autowired
    private UserRoleMapper userRoleMapper;

    @Autowired
    private UserResourceService userResourceService;

    @Autowired
    private RoleManagerService roleManagerService;

    /**
     * 通过resourceCode列表查询resource对象列表
     * @param codeList
     * @return
     */
    @Override
    public List<UserResource> listUserResourceByCodes(List<String> codeList) {
        QueryWrapper<UserResource> userResourceQw = new QueryWrapper<>();
        userResourceQw.in("resource_code",codeList);
        userResourceQw.eq("is_delete",0);
        List<UserResource> userResourceList = userResourceMapper.selectList(userResourceQw);

        return userResourceList;
    }

    /**
     * 通过资源code获取资源对象
     * @param resourceCode
     * @return
     */
    @Override
    public UserResource getUserResourceByCode(String resourceCode) {
        QueryWrapper<UserResource> userResourceQw = new QueryWrapper<>();
        userResourceQw.eq("resource_code",resourceCode);
        userResourceQw.eq("is_delete",0);
        UserResource userResource = userResourceMapper.selectOne(userResourceQw);

        return userResource;
    }

    /**
     * 保存角色资源
     * @param roleCode
     * @param resourceCodeList
     */
    @Override
    public void saveUserResourceMapping(String roleCode, List<String> resourceCodeList) {
        //查询角色
        UserRole role = roleManagerService.getRoleByCode(roleCode);

        //1-查询用户现有资源列表
        List<String> curResourceCodeList = listResourceByRoleCode(roleCode);

        //2-比较用户现有资源与入参传入的目标资源，整理出-->需删除资源 and 需增加资源
        List<String> needDeleteResourceList = ICollectionUtil.differenceList(curResourceCodeList, resourceCodeList);
        List<String> needAddResourceList = ICollectionUtil.differenceList(resourceCodeList, curResourceCodeList);

        //3-批量删除需删除资源
        if(CollectionUtils.isNotEmpty(needDeleteResourceList)){
            //查询该角色以及其下的角色
            List<UserRole> userRoleList = userRoleMapper.listRoleTree(role);
            //循环删除子角色的这些resource
            for(UserRole tempRole:userRoleList){
                userResourceService.deleteResourceMapping(tempRole.getRoleCode(),needDeleteResourceList);
            }
        }

        //4-批量添加需增加资源
        if(CollectionUtils.isNotEmpty(needAddResourceList)){
            //先查询对象列表
            List<UserResource> needAddResourceEntityList = listUserResourceByCodes(needAddResourceList);
            //构造待添加的mapping对象列表
            List<UserMappingRoleResource> roleResourceList = new ArrayList<>();
            for(UserResource resource:needAddResourceEntityList){
                UserMappingRoleResource roleResource = new UserMappingRoleResource();
                roleResource.setRoleCode(roleCode);
                roleResource.setResourceCode(resource.getResourceCode());
                roleResource.setResourceName(resource.getResourceName());
                roleResourceList.add(roleResource);
            }
            //批量保存
            saveBatch(roleResourceList);
        }
    }

    @Override
    public void deleteResourceMapping(String roleCode, List<String> needDeleteResourceList) {
        UpdateWrapper<UserMappingRoleResource> userMappingRoleResourceUw = new UpdateWrapper<>();
        userMappingRoleResourceUw
                //where
                .eq("role_code",roleCode)
                .in("resource_code",needDeleteResourceList)
                //set
                .set("is_delete",1);
        userMappingRoleResourceMapper.update(null,userMappingRoleResourceUw);
    }

    /**
     * 查询角色名下所有资源
     * @param roleCode
     * @return
     */
    @Override
    public List<String> listResourceByRoleCode(String roleCode) {
        //1-查询角色的关联资源
        QueryWrapper<UserMappingRoleResource> userMappingRoleResourceQw = new QueryWrapper<>();
        userMappingRoleResourceQw.eq("role_code",roleCode);
        userMappingRoleResourceQw.eq("is_delete",0);
        List<UserMappingRoleResource> userMappingRoleResourceList = userMappingRoleResourceMapper.selectList(userMappingRoleResourceQw);

        //2-转化出参
        List<String> resourceCodeList = userMappingRoleResourceList.stream().map(map -> map.getResourceCode()).collect(Collectors.toList());

        return resourceCodeList;
    }



}
